def primes(nb_primes):
    p = []
    n = 2
    while len(p) < nb_primes:
        # Is n prime?
        for i in p:
            if n % i == 0:
                break

        # If no break occurred in the loop
        else:
            p.append(n)
        n += 1
    return p



# mode: run
# tag: numpy

cimport cython

import numpy as np
cimport numpy as cnp

cnp.import_array()


@cython.test_assert_path_exists(
    "//ReturnStatNode",
    "//ReturnStatNode//IndexNode",
    "//ReturnStatNode//IndexNode//SimpleCallNode",
)
@cython.test_fail_if_path_exists(
    "//ReturnStatNode//AttributeNode",
)
def access_shape():
    """
    >>> print(access_shape())
    10
    """
    cdef cnp.ndarray[double, ndim=2, mode='c'] array_in = \
        1e10 * np.ones((10, 10))

    return array_in.shape[0]


@cython.test_assert_path_exists(
    "//ReturnStatNode",
    "//ReturnStatNode//SimpleCallNode",
)
@cython.test_fail_if_path_exists(
    "//ReturnStatNode//AttributeNode",
)
def access_size():
    """
    >>> print(access_size())
    100
    """
    cdef cnp.ndarray[double, ndim=2, mode='c'] array_in = \
        1e10 * np.ones((10, 10))

    return array_in.size


@cython.test_assert_path_exists(
    "//ReturnStatNode",
    "//ReturnStatNode//IndexNode",
    "//ReturnStatNode//IndexNode//SimpleCallNode",
)
@cython.test_fail_if_path_exists(
    "//ReturnStatNode//AttributeNode",
)
def access_strides():
    """
    >>> x, y = access_strides()
    >>> print(x)
    80
    >>> print(y)
    8
    """
    cdef cnp.ndarray[double, ndim=2, mode='c'] array_in = \
        1e10 * np.ones((10, 10), dtype=np.float64)

    return (array_in.strides[0], array_in.strides[1])


@cython.test_assert_path_exists(
    "//ReturnStatNode",
    "//ReturnStatNode//PrimaryCmpNode",
    "//ReturnStatNode//PrimaryCmpNode//SimpleCallNode",
)
@cython.test_fail_if_path_exists(
    "//ReturnStatNode//AttributeNode",
)
def access_data():
    """
    >>> access_data()
    True
    """
    cdef cnp.ndarray[double, ndim=2, mode='c'] array_in = \
        1e10 * np.ones((10, 10), dtype=np.float64)

    return array_in.data is not NULL



from .object cimport PyObject
from .object cimport PyTypeObject, Py_TYPE  # legacy imports for re-export

cdef extern from "Python.h":
    #####################################################################
    # 3. Reference Counts
    #####################################################################
    # The macros in this section are used for managing reference counts of Python objects.
    void Py_INCREF(object o)
    # Increment the reference count for object o. The object must not
    # be NULL; if you aren't sure that it isn't NULL, use
    # Py_XINCREF().

    void Py_XINCREF(PyObject* o)
    # Increment the reference count for object o. The object may be NULL, in which case the macro has no effect.

    void Py_DECREF(object o)
    # Decrement the reference count for object o. The object must not
    # be NULL; if you aren't sure that it isn't NULL, use
    # Py_XDECREF(). If the reference count reaches zero, the object's
    # type's deallocation function (which must not be NULL) is
    # invoked.

    # Warning: The deallocation function can cause arbitrary Python
    # code to be invoked (e.g. when a class instance with a __del__()
    # method is deallocated). While exceptions in such code are not
    # propagated, the executed code has free access to all Python
    # global variables. This means that any object that is reachable
    # from a global variable should be in a consistent state before
    # Py_DECREF() is invoked. For example, code to delete an object
    # from a list should copy a reference to the deleted object in a
    # temporary variable, update the list data structure, and then
    # call Py_DECREF() for the temporary variable.

    void Py_XDECREF(PyObject* o)
    # Decrement the reference count for object o. The object may be
    # NULL, in which case the macro has no effect; otherwise the
    # effect is the same as for Py_DECREF(), and the same warning
    # applies.

    void Py_CLEAR(PyObject* o)
    # Decrement the reference count for object o. The object may be
    # NULL, in which case the macro has no effect; otherwise the
    # effect is the same as for Py_DECREF(), except that the argument
    # is also set to NULL. The warning for Py_DECREF() does not apply
    # with respect to the object passed because the macro carefully
    # uses a temporary variable and sets the argument to NULL before
    # decrementing its reference count.
    # It is a good idea to use this macro whenever decrementing the
    # value of a variable that might be traversed during garbage
    # collection.

    Py_ssize_t Py_REFCNT(object o)
    # Get the reference count of the Python object o.

    # Note that the returned value may not actually reflect how many
    # references to the object are actually held. For example, some
    # objects are “immortal” and have a very high refcount that does not
    # reflect the actual number of references. Consequently, do not rely
    # on the returned value to be accurate, other than a value of 0 or
    # 1.

    Py_ssize_t _Py_REFCNT "Py_REFCNT" (PyObject *ptr)
    # Get the reference count for the PyObject pointer ptr.

    # This is useful when it would be awkward to create an owned reference just
    # to get the reference count. See the note for Py_REFCNT above about the
    # accuracy of reference counts.

    int PyUnstable_Object_EnableDeferredRefcount(object o)
    # Enable deferred reference counting on obj, if supported by the runtime.
    # In the free-threaded build, this allows the interpreter to avoid reference count
    # adjustments to obj, which may improve multi-threaded performance.
    # The tradeoff is that obj will only be deallocated by the tracing garbage collector,
    # and not when the interpreter no longer has any references to it.
    #
    # This function returns 1 if deferred reference counting is enabled on obj,
    # and 0 if deferred reference counting is not supported or if the hint was ignored
    # by the interpreter, such as when deferred reference counting is already enabled on obj.
    # This function is thread-safe, and cannot fail.
    #
    # This function does nothing on builds with the GIL enabled, which do not support
    # deferred reference counting. This also does nothing if obj is not an object tracked
    # by the garbage collector (see gc.is_tracked() and PyObject_GC_IsTracked()).
    #
    # This function is intended to be used soon after obj is created, by the code
    # that creates it, such as in the object’s tp_new slot.
    #
    # Added in CPython 3.14.

    int PyUnstable_Object_IsUniqueReferencedTemporary(object o)
    # Check if obj is a unique temporary object.
    # Returns 1 if obj is known to be a unique temporary object, and 0 otherwise.
    # This function cannot fail, but the check is conservative, and may return 0
    # in some cases even if obj is a unique temporary object.
    #
    # If an object is a unique temporary, it is guaranteed that the current code
    # has the only reference to the object. For arguments to C functions, this should
    # be used instead of checking if the reference count is 1.
    # Starting with Python 3.14, the interpreter internally avoids some reference count
    # modifications when loading objects onto the operands stack by borrowing references
    # when possible, which means that a reference count of 1 by itself does not guarantee
    # that a function argument uniquely referenced.
    #
    # Added in CPython 3.14.

    int PyUnstable_IsImmortal(object o)
    # This function returns non-zero if obj is immortal, and zero otherwise.
    # This function cannot fail.
    #
    # Added in CPython 3.14.

    void PyUnstable_EnableTryIncRef(object o)
    # Enables subsequent uses of PyUnstable_TryIncRef() on obj.
    # The caller must hold a strong reference to obj when calling this.
    #
    # Added in CPython 3.14.

    bint PyUnstable_TryIncRef(PyObject *o)
    # Increments the reference count of obj if it is not zero.
    # Returns 1 if the object’s reference count was successfully incremented.
    # Otherwise, this function returns 0.
    #
    # PyUnstable_EnableTryIncRef() must have been called earlier on obj
    # or this function may spuriously return 0 in the free threading build.
    #
    # Added in CPython 3.14.

    int PyUnstable_Object_IsUniquelyReferenced(object o)
    # Determine if op only has one reference.
    #
    # On GIL-enabled builds, this function is equivalent to Py_REFCNT(op) == 1.
    #
    # On a free threaded build, this checks if op’s reference count is equal
    # to one and additionally checks if op is only used by this thread.
    # Py_REFCNT(op) == 1 is not thread-safe on free threaded builds; prefer this function.
    #
    # The caller must hold an attached thread state, despite the fact that this function
    # doesn’t call into the Python interpreter. This function cannot fail.
    #
    # Added in CPython 3.14.



# ticket: t248
# mode: compile

from ewing8 cimport (Foo,


                     Blarg)

from declandimpl cimport (Sandwich


                          ,




                          Tomato)

cdef extern Foo yummy



# cython: remove_unreachable=False
# mode: error
import cython

@cython.nogil
@cython.cfunc
def f_nogil(x: cython.int) -> cython.int:
    y: cython.int
    y = x + 10
    return y


def f_gil(x):
    y = 0
    y = x + 100
    return y


def illegal_gil_usage():
    res: cython.int = 0
    with cython.nogil(True):
        res = f_gil(res)

        with cython.nogil(True):
            res = f_gil(res)

    with cython.nogil(False):
        res = f_nogil(res)


def foo(a):
    return a < 10


def non_constant_condition(x: cython.int) -> cython.int:
    res: cython.int = x
    with cython.nogil(x < 10):
        res = f_nogil(res)


number_or_object = cython.fused_type(cython.float, cython.object)


def fused_type(x: number_or_object):
    with cython.nogil(number_or_object is object):
        res = x + 1

    # This should be fine
    with cython.nogil(number_or_object is cython.float):
        res = x + 1

    return res

def nogil_multiple_arguments(x: cython.int) -> cython.int:
    res: cython.int = x
    with cython.nogil(1, 2):
        res = f_nogil(res)

def nogil_keyworkd_arguments(x: cython.int) -> cython.int:
    res: cython.int = x
    with cython.nogil(kw=2):
        res = f_nogil(res)


@cython.gil(True)
@cython.cfunc
def wrong_decorator() -> cython.int:
    return 0


_ERRORS = u"""
22:14: Accessing Python global or builtin not allowed without gil
22:19: Calling gil-requiring function not allowed without gil
22:19: Coercion from Python not allowed without the GIL
22:19: Constructing Python tuple not allowed without gil
22:20: Converting to Python object not allowed without gil
24:13: Trying to release the GIL while it was previously released.
25:18: Accessing Python global or builtin not allowed without gil
25:23: Calling gil-requiring function not allowed without gil
25:23: Coercion from Python not allowed without the GIL
25:23: Constructing Python tuple not allowed without gil
25:24: Converting to Python object not allowed without gil
37:24: Non-constant condition in a `with nogil(<condition>)` statement
46:8: Assignment of Python object not allowed without gil
46:16: Calling gil-requiring function not allowed without gil
56:9: Compiler directive nogil accepts one positional argument.
61:9: Compiler directive nogil accepts one positional argument.
65:0: The gil compiler directive is not allowed in function scope
"""



 
cdef class Animal:

    cdef int number_of_legs
    cdef dict __dict__

    def __init__(self, int number_of_legs):
        self.number_of_legs = number_of_legs


dog = Animal(4)
dog.has_tail = True



# mode: run
# tag: cpp, werror, cpp20

from libcpp.numeric cimport midpoint

def test_midpoint_integer(int a, int b):
    """
    Test midpoint for integer types
    >>> test_midpoint_integer(2, 6)
    4
    """
    cdef int res = midpoint[int](a, b)
    return res


def test_midpoint_float(float a, float b):
    """
    Test midpoint for float
    >>> test_midpoint_float(2, 6)
    4.0
    """
    cdef float res = midpoint[float](a, b)
    return res



from cython.cimports.cpython.mem import PyMem_Malloc, PyMem_Realloc, PyMem_Free

@cython.cclass
class SomeMemory:
    data: cython.p_double

    def __cinit__(self, number: cython.size_t):
        # allocate some memory (uninitialised, may contain arbitrary data)
        self.data = cython.cast(cython.p_double, PyMem_Malloc(
            number * cython.sizeof(cython.double)))
        if not self.data:
            raise MemoryError()

    def resize(self, new_number: cython.size_t):
        # Allocates new_number * sizeof(double) bytes,
        # preserving the current content and making a best-effort to
        # reuse the original data location.
        mem = cython.cast(cython.p_double, PyMem_Realloc(
            self.data, new_number * cython.sizeof(cython.double)))
        if not mem:
            raise MemoryError()
        # Only overwrite the pointer if the memory was really reallocated.
        # On error (mem is NULL), the originally memory has not been freed.
        self.data = mem

    def __dealloc__(self):
        PyMem_Free(self.data)  # no-op if self.data is NULL



# tag: posix

from posix.unistd cimport *
from posix.resource cimport *


def test_getpriority():
    """
    >>> test_getpriority()
    0
    """
    ret = getpriority(PRIO_PROCESS, getpid())
    # DISABLED - does not work on current test server
    return 0  # ret


def test_getrlimit():
    """
    >>> test_getrlimit()
    0
    True
    """
    cdef rlimit rlim
    rlim.rlim_cur = 0

    ret = getrlimit(RLIMIT_CPU, &rlim)
    print(ret)
    return rlim.rlim_cur != 0


def test_getrusage():
    """
    >>> test_getrusage()
    0
    """
    cdef rusage r
    ret = getrusage(RUSAGE_SELF, &r)
    return ret



# mode: run
# tag: cpp, cpp20, no-cpp-locals

from libcpp cimport stop_token
import threading

def test_basic_usage1():
    """
    >>> test_basic_usage1()
    """
    ss = stop_token.stop_source()
    assert not ss.stop_requested()
    assert ss.stop_possible()

    st = ss.get_token()
    assert not st.stop_requested()
    assert st.stop_possible()

    ss.request_stop()
    assert st.stop_requested()
    assert ss.stop_requested()

def test_basic_usage2():
    """
    >>> test_basic_usage2()
    """
    ss = stop_token.stop_source(stop_token.nostopstate)
    assert not ss.stop_requested()
    assert not ss.stop_possible()
    
    ss.request_stop()
    assert not ss.stop_requested()

cdef int global_flag = 0

cdef void use_as_stop_callback() nogil noexcept:
    global_flag = 1

def test_cfunc_stop_callback():
    """
    >>> test_cfunc_stop_callback()
    """
    global global_flag

    global_flag = 1

    ss = stop_token.stop_source()
    
    def other_thread():
        ss.request_stop()

    # stop callback is hard to use except on the heap because it can't be
    # default constructed or assigned
    cb = new stop_token.func_ptr_stop_callback(ss.get_token(), use_as_stop_callback)
    try:
        t = threading.Thread(target=other_thread)
        t.start()
        t.join()

        assert global_flag
    finally:
        del cb

def test_python_stop_callback_wrapper():
    """
    >>> test_python_stop_callback_wrapper()
    """
    cdef stop_token.python_stop_callback_holder cb_holder
    ss = stop_token.stop_source()

    local_flag = False

    def callback_func():
        nonlocal local_flag
        local_flag = True

    cb_holder.initialize(ss.get_token(), callback_func)

    def other_thread():
        ss.request_stop()

    t = threading.Thread(target=other_thread)
    t.start()
    t.join()

    assert local_flag



# ticket: t676
# tag: cpp

from cython cimport typeof

cdef extern from "arithmetic_analyse_types_helper.h":
    cdef struct short_return:
        char *msg
    cdef struct int_return:
        char *msg
    cdef struct longlong_return:
        char *msg
    cdef short_return f(short)
    cdef int_return f(int)
    cdef longlong_return f(long long)

def short_binop(short val):
    """
    Arithmetic in C is always done with at least int precision.
    
    >>> print(short_binop(3))
    int called
    """
    assert typeof(val + val) == "int", typeof(val + val)
    assert typeof(val - val) == "int", typeof(val - val)
    assert typeof(val & val) == "int", typeof(val & val)
    cdef int_return x = f(val + val)
    return x.msg.decode('ASCII')

def short_unnop(short val):
    """
    Arithmetic in C is always done with at least int precision.
    
    >>> print(short_unnop(3))
    int called
    """
    cdef int_return x = f(-val)
    return x.msg.decode('ASCII')

def longlong_binop(long long val):
    """
    >>> print(longlong_binop(3))
    long long called
    """
    cdef longlong_return x = f(val * val)
    return x.msg.decode('ASCII')

def longlong_unnop(long long val):
    """
    >>> print(longlong_unnop(3))
    long long called
    """
    cdef longlong_return x = f(~val)
    return x.msg.decode('ASCII')


def test_bint(bint a):
    """
    >>> test_bint(True)
    """
    assert typeof(a + a) == "int", typeof(a + a)
    assert typeof(a & a) == "bint", typeof(a & a)



# mode: run
# ticket: t470


def func(**kwargs):
    """
    >>> func(**{'a' : 7})
    True
    >>> func(**SubDict())
    True
    >>> func(**NonDict())
    True
    """
    return type(kwargs) is dict and kwargs['a'] == 7


class NonDict(object):
   def __getitem__(self, k):
       assert k == 'a'
       return 7
   def keys(self):
       return ['a']

def call_non_dict_test():
    """
    >>> call_non_dict_test()
    True
    """
    return func(**NonDict())

def call_non_dict_test_kw():
    """
    >>> call_non_dict_test_kw()
    True
    """
    return func(b=5, **NonDict())


class SubDict(dict):
    def __init__(self):
        self['a'] = 7

def call_sub_dict_test():
    """
    >>> call_sub_dict_test()
    True
    """
    return func(**SubDict())

def call_sub_dict_test_kw():
    """
    >>> call_sub_dict_test_kw()
    True
    """
    return func(b=5, **SubDict())



from cython cimport view

def main():
    # direct access in both dimensions, strided in the first dimension, contiguous in the last
    cdef int[:, ::view.contiguous] a

    # contiguous list of pointers to contiguous lists of ints
    cdef int[::view.indirect_contiguous, ::1] b

    # direct or indirect in the first dimension, direct in the second dimension
    # strided in both dimensions
    cdef int[::view.generic, :] c



# mode: run
# ticket: t768
from cython cimport typeof

def type_inference_del_int():
    """
    >>> type_inference_del_int()
    'Python object'
    """
    x = 1
    del x
    return typeof(x)

def type_inference_del_dict():
    """
    >>> type_inference_del_dict()
    'dict object'
    """
    x = {}
    del x
    return typeof(x)



# mode: error
# tag: cpp, warnings

cdef inline void handle_exception():
    pass

# GH 3064 - cppfunc caused invalid code to be generated with +handle_exception
# error to prevent this
cdef test_func1(self) except +handle_exception:
    pass

# warning
cdef test_func2(self) except +:
    pass


_ERRORS = """
9:0: Only extern functions can throw C++ exceptions.
"""

_WARNINGS = """
13:0: Only extern functions can throw C++ exceptions.
"""



# mode: compile

# the following are valid syntax constructs and should not produce errors

ctypedef int x;

cdef no_semi():
    cdef int i

cdef with_semi():
    cdef int i;

def use_cdef():
  &no_semi, &with_semi



__doc__ = u"""
>>> import sys
>>> sys.getrefcount(Foo.__pyx_vtable__)
2
>>> sys.getrefcount(__pyx_capi__['bar'])
2
>>> sys.getrefcount(__pyx_capi__['spam'])
2
>>> sys.getrefcount(__pyx_capi__['ten'])
2
>>> sys.getrefcount(__pyx_capi__['pi'])
2
>>> sys.getrefcount(__pyx_capi__['obj'])
2
>>> sys.getrefcount(__pyx_capi__['dct'])
2
>>> sys.getrefcount(__pyx_capi__['tpl'])
2
>>> sys.getrefcount(__pyx_capi__['one'])
2
>>> sys.getrefcount(__pyx_capi__['two'])
Traceback (most recent call last):
  ...
KeyError: 'two'
"""

cdef public api class Foo [type FooType, object FooObject]:
    cdef void bar(self):
        pass

cdef public api void bar():
    pass
cdef api void spam():
    pass

cdef api int    ten = 10
cdef api double pi = 3.14
cdef api object obj = object()
cdef api dict   dct = {}

cdef public api tuple tpl = ()
cdef public api float one = 1
cdef public     float two = 2




from libc.stdlib cimport free
from c_func cimport c_call_returning_a_c_string

cdef bytes py_string
cdef char* c_string = c_call_returning_a_c_string()
try:
    py_string = c_string
finally:
    free(c_string)



# mode: run
# tag: exttype, final

cimport cython

cdef class TopBase:
    cdef method(self):
        return None


cdef class BaseClass(TopBase):
    """
    >>> obj = BaseClass()
    >>> obj.call_base()
    'base'
    """
    cdef base(self):
        return '@base'

    cdef method(self):
        return 'base'

    def call_base(self):
        return self.method()


@cython.final
cdef class ChildClass(BaseClass):
    """
    >>> obj = ChildClass()
    >>> obj.call_base()
    'child'
    >>> obj.call_child()
    'child'
    """
    cdef child(self):
        return '@child'

    cdef method(self):
        return 'child'

    def call_child(self):
        # original bug: this requires a proper cast for self
        return self.method()


def test_BaseClass():
    """
    >>> test_BaseClass()
    '@base'
    'base'
    """
    cdef BaseClass obj = BaseClass()
    print(repr(obj.base()))
    print(repr(obj.method()))


def test_ChildClass():
    """
    >>> test_ChildClass()
    '@base'
    '@child'
    'child'
    '@base'
    '@child'
    'child'
    """
    cdef ChildClass obj = ChildClass()
    print(repr(obj.base()))
    print(repr(obj.child()))
    print(repr(obj.method()))
    cdef BaseClass bobj = obj
    print(repr(obj.base()))
    print(repr(obj.child()))
    print(repr(bobj.method()))




def f():
    """
    >>> f()
    >>> g
    42
    >>> x == 'spam'
    True
    >>> y == 'eggs'
    True
    >>> z == 'spameggs'
    True
    """
    pass

g = 42
x = u"spam"
y = u"eggs"
if g:
    z = x + y



# mode: run
# tag: pep484, warnings
# cython: language_level=3

cimport cython
from cython cimport typeof
from cpython.ref cimport PyObject

try:
    from typing import Optional, Union
except ImportError:
    pass


def old_dict_syntax(a: list, b: "int" = 2, c: {'ctype': 'long int'} = 3, d: {'type': 'long int'} = 4) -> list:
    """
    >>> old_dict_syntax([1])
    ('list object', 'int object', 'long', 'long')
    [1, 2, 3, 4]
    >>> old_dict_syntax([1], 3)
    ('list object', 'int object', 'long', 'long')
    [1, 3, 3, 4]
    >>> old_dict_syntax(123)
    Traceback (most recent call last):
    TypeError: Argument 'a' has incorrect type (expected list, got int)
    >>> old_dict_syntax(None)
    Traceback (most recent call last):
    TypeError: Argument 'a' has incorrect type (expected list, got NoneType)
    """
    print((typeof(a), typeof(b), typeof(c), typeof(d)))
    a.append(b)
    a.append(c)
    a.append(d)
    return a


def pytypes_def(a: list, b: int = 2, c: long = 3, d: float = 4.0, n: list = None, o: Optional[tuple] = (), v: Union[tuple, None]=(), u: tuple | None = ()) -> list:
    """
    >>> pytypes_def([1])
    ('list object', 'int object', 'Python object', 'double', 'list object', 'tuple object', 'tuple object', 'tuple object')
    [1, 2, 3, 4.0, None, (), (), ()]
    >>> pytypes_def([1], 3)
    ('list object', 'int object', 'Python object', 'double', 'list object', 'tuple object', 'tuple object', 'tuple object')
    [1, 3, 3, 4.0, None, (), (), ()]
    >>> pytypes_def([1], 3, 2, 1, [], None, None, None)
    ('list object', 'int object', 'Python object', 'double', 'list object', 'tuple object', 'tuple object', 'tuple object')
    [1, 3, 2, 1.0, [], None, None, None]
    >>> pytypes_def([1], 3, 2, 1, [], 'a', (), ())
    Traceback (most recent call last):
    TypeError: Argument 'o' has incorrect type (expected tuple, got str)
    >>> pytypes_def([1], 3, 2, 1, [], (), 'a')
    Traceback (most recent call last):
    TypeError: Argument 'v' has incorrect type (expected tuple, got str)
    >>> pytypes_def([1], 3, 2, 1, [], (), (), 'a')
    Traceback (most recent call last):
    TypeError: Argument 'u' has incorrect type (expected tuple, got str)
    >>> pytypes_def(123)
    Traceback (most recent call last):
    TypeError: Argument 'a' has incorrect type (expected list, got int)
    >>> pytypes_def(None)
    Traceback (most recent call last):
    TypeError: Argument 'a' has incorrect type (expected list, got NoneType)
    """
    print((typeof(a), typeof(b), typeof(c), typeof(d), typeof(n), typeof(o), typeof(v), typeof(u)))
    a.append(b)
    a.append(c)
    a.append(d)
    a.append(n)
    a.append(o)
    a.append(v)
    a.append(u)
    return a


cpdef pytypes_cpdef(a: list, b: int = 2, c: long = 3, d: float = 4.0, n: list = None, o: Optional[tuple] = (), v: Union[tuple, None]=(), u: tuple | None = ()):
    """
    >>> pytypes_cpdef([1])
    ('list object', 'int object', 'Python object', 'double', 'list object', 'tuple object', 'tuple object', 'tuple object')
    [1, 2, 3, 4.0, None, (), (), ()]
    >>> pytypes_cpdef([1], 3)
    ('list object', 'int object', 'Python object', 'double', 'list object', 'tuple object', 'tuple object', 'tuple object')
    [1, 3, 3, 4.0, None, (), (), ()]
    >>> pytypes_cpdef([1], 3, 2, 1, [], None, None, None)
    ('list object', 'int object', 'Python object', 'double', 'list object', 'tuple object', 'tuple object', 'tuple object')
    [1, 3, 2, 1.0, [], None, None, None]
    >>> pytypes_cpdef([1], 3, 2, 1, [], 'a', (), ())
    Traceback (most recent call last):
    TypeError: Argument 'o' has incorrect type (expected tuple, got str)
    >>> pytypes_cpdef([1], 3, 2, 1, [], (), 'a', ())
    Traceback (most recent call last):
    TypeError: Argument 'v' has incorrect type (expected tuple, got str)
    >>> pytypes_cpdef([1], 3, 2, 1, [], (), (), 'a')
    Traceback (most recent call last):
    TypeError: Argument 'u' has incorrect type (expected tuple, got str)
    >>> pytypes_cpdef(123)
    Traceback (most recent call last):
    TypeError: Argument 'a' has incorrect type (expected list, got int)
    >>> pytypes_cpdef(None)
    Traceback (most recent call last):
    TypeError: Argument 'a' has incorrect type (expected list, got NoneType)
    """
    print((typeof(a), typeof(b), typeof(c), typeof(d), typeof(n), typeof(o), typeof(v), typeof(u)))
    a.append(b)
    a.append(c)
    a.append(d)
    a.append(n)
    a.append(o)
    a.append(v)
    a.append(u)
    return a


cdef c_pytypes_cdef(a: list, b: int = 2, c: long = 3, d: float = 4.0, n: list = None):
    print((typeof(a), typeof(b), typeof(c), typeof(d), typeof(n)))
    a.append(b)
    a.append(c)
    a.append(d)
    a.append(n)
    return a


def pytypes_cdef(a, b=2, c=3, d=4):
    """
    >>> pytypes_cdef([1])
    ('list object', 'int object', 'Python object', 'double', 'list object')
    [1, 2, 3, 4.0, None]
    >>> pytypes_cdef([1], 3)
    ('list object', 'int object', 'Python object', 'double', 'list object')
    [1, 3, 3, 4.0, None]
    >>> pytypes_cdef(123)   # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...
    """
    return c_pytypes_cdef(a, b, c, d)


def pyint(a: int):
    """
    >>> large_int = eval('0x'+'F'*64)  # definitely bigger than C int64
    >>> pyint(large_int) == large_int
    True
    """
    return a


def ctypes_def(a: list, b: cython.int = 2, c: cython.long = 3, d: cython.float = 4) -> list:
    """
    >>> ctypes_def([1])
    ('list object', 'int', 'long', 'float')
    [1, 2, 3, 4.0]
    >>> ctypes_def([1], 3)
    ('list object', 'int', 'long', 'float')
    [1, 3, 3, 4.0]
    >>> ctypes_def(123)
    Traceback (most recent call last):
    TypeError: Argument 'a' has incorrect type (expected list, got int)
    """
    print((typeof(a), typeof(b), typeof(c), typeof(d)))
    a.append(b)
    a.append(c)
    a.append(d)
    return a


def return_tuple_for_carray() -> tuple:
    """
    >>> return_tuple_for_carray()
    (1, 2, 3)
    """
    cdef int[3] x
    x = [1, 2, 3]
    return x


def invalid_ctuple_syntax(a: (cython.int, cython.int), b: (int, int)):
    """
    >>> invalid_ctuple_syntax([1, 2], [3, 4])
    [1, 2, 3, 4]
    """
    result: (cython.int, cython.int, cython.int, cython.int) = a + b
    return result


MyStruct = cython.struct(x=cython.int, y=cython.int, data=cython.double)


@cython.ccall
def struct_io(s : MyStruct) -> MyStruct:
    """
    >>> d = struct_io(dict(x=1, y=2, data=3))
    >>> sorted(d.items())
    [('data', 3.0), ('x', 2), ('y', 1)]
    >>> d = struct_io(None)
    Traceback (most recent call last):
    TypeError: Expected a mapping, got NoneType
    """
    t = s
    t.x, t.y = s.y, s.x
    return t


@cython.test_fail_if_path_exists(
    "//CoerceFromPyTypeNode",
    "//SimpleCallNode//CoerceToPyTypeNode",
)
@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//SimpleCallNode",
)
def call_struct_io(s : MyStruct) -> MyStruct:
    """
    >>> d = call_struct_io(dict(x=1, y=2, data=3))
    >>> sorted(d.items())
    [('data', 3.0), ('x', 2), ('y', 1)]
    >>> d = call_struct_io(None)
    Traceback (most recent call last):
    TypeError: Expected a mapping, got NoneType
    """
    return struct_io(s)


@cython.test_assert_path_exists(
    "//CFuncDefNode",
    "//CFuncDefNode//DefNode",
    "//CFuncDefNode[@return_type]",
    "//CFuncDefNode[@return_type.is_struct_or_union = True]",
)
@cython.ccall
def struct_convert(d) -> MyStruct:
    """
    >>> d = struct_convert(dict(x=1, y=2, data=3))
    >>> sorted(d.items())
    [('data', 3.0), ('x', 1), ('y', 2)]
    >>> struct_convert({})  # make sure we can raise exceptions through struct return values
    Traceback (most recent call last):
    ValueError: No value specified for struct attribute 'x'
    """
    return d


@cython.test_assert_path_exists(
    "//CFuncDefNode",
    "//CFuncDefNode//DefNode",
    "//CFuncDefNode[@return_type]",
    "//CFuncDefNode[@return_type.is_int = True]",
)
@cython.ccall
def exception_default(raise_exc : cython.bint = False) -> cython.int:
    """
    >>> exception_default(raise_exc=False)
    10
    >>> exception_default(raise_exc=True)
    Traceback (most recent call last):
    ValueError: huhu!
    """
    if raise_exc:
        raise ValueError("huhu!")
    return 10


def call_exception_default(raise_exc=False):
    """
    >>> call_exception_default(raise_exc=False)
    10
    >>> call_exception_default(raise_exc=True)
    Traceback (most recent call last):
    ValueError: huhu!
    """
    return exception_default(raise_exc)


@cython.test_assert_path_exists(
    "//CFuncDefNode",
    "//CFuncDefNode//DefNode",
    "//CFuncDefNode[@return_type]",
    "//CFuncDefNode[@return_type.is_int = True]",
)
@cython.ccall
def exception_default_uint(raise_exc : cython.bint = False) -> cython.uint:
    """
    >>> print(exception_default_uint(raise_exc=False))
    10
    >>> exception_default_uint(raise_exc=True)
    Traceback (most recent call last):
    ValueError: huhu!
    """
    if raise_exc:
        raise ValueError("huhu!")
    return 10


def call_exception_default_uint(raise_exc=False):
    """
    >>> print(call_exception_default_uint(raise_exc=False))
    10
    >>> call_exception_default_uint(raise_exc=True)
    Traceback (most recent call last):
    ValueError: huhu!
    """
    return exception_default_uint(raise_exc)


class EarlyClass(object):
    """
    >>> a = EarlyClass(1)
    >>> a.string_forward_declaration()  # should probably raise an error at some point
    1
    >>> x = LateClass()
    >>> a = EarlyClass(x)
    >>> x2 = a.string_forward_declaration()
    >>> assert x is x2, x2
    """
    def __init__(self, x):
        self.x = x
    def string_forward_declaration(self) -> 'LateClass':
        return self.x

class LateClass(object):
    pass


def py_float_default(price : Optional[float]=None, bar: Union[float, None]=None, spam: float | None = None, ndigits=4):
    """
    Python default arguments should prevent C type inference.

    >>> py_float_default()
    (None, None, None, 4)
    >>> py_float_default(None)
    (None, None, None, 4)
    >>> py_float_default(2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...float...
    >>> py_float_default(2.0, 3.0, 4.0)
    (2.0, 3.0, 4.0, 4)
    >>> py_float_default(2, None, None, 4)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...float...
    >>> py_float_default(None, 2, None, 4)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...float...
    """
    return price, bar, spam, ndigits


cdef class ClassAttribute:
    cls_attr : cython.float = 1.


@cython.cfunc
def take_ptr(obj: cython.pointer(PyObject)):
    pass

def call_take_ptr():
    """
    >>> call_take_ptr()  # really just a compile-test
    """
    python_dict = {"abc": 123}
    take_ptr(cython.cast(cython.pointer(PyObject), python_dict))

@cython.cclass
class HasPtr:
    """
    >>> HasPtr()
    HasPtr(1, 1)
    """
    a: cython.pointer(cython.int)
    b: cython.int

    def __init__(self):
        self.b = 1
        self.a = cython.address(self.b)
    def __repr__(self):
        return f"HasPtr({self.a[0]}, {self.b})"


@cython.annotation_typing(False)
def turn_off_typing(x: float, d: dict):
    """
    >>> turn_off_typing('not a float', [])  # ignore the typing
    ('Python object', 'Python object', 'not a float', [])
    """
    return typeof(x), typeof(d), x, d


@cython.annotation_typing(False)
cdef class ClassTurnOffTyping:
    x: float
    d: dict

    def get_var_types(self, arg: float):
        """
        >>> ClassTurnOffTyping().get_var_types(1.0)
        ('Python object', 'Python object', 'Python object')
        """
        return typeof(self.x), typeof(self.d), typeof(arg)

    @cython.annotation_typing(True)
    def and_turn_it_back_on_again(self, arg: float):
        """
        >>> ClassTurnOffTyping().and_turn_it_back_on_again(1.0)
        ('Python object', 'Python object', 'double')
        """
        return typeof(self.x), typeof(self.d), typeof(arg)


from cython cimport int as cy_i


def int_alias(a: cython.int, b: cy_i):
    """
    >>> int_alias(1, 2)
    int
    int
    """
    print(cython.typeof(a))
    print(cython.typeof(b))


def test_inexact_types(d: dict):
    """
    >>> test_inexact_types({})  # good

    Check that our custom pep484 warning is in either the error message
    or the exception notes
    >>> from collections import OrderedDict
    >>> try:
    ...    test_inexact_types(OrderedDict())
    ... except TypeError as e:
    ...    assert ("Cython is deliberately stricter than PEP-484" in e.args[0] or
    ...            any("Cython is deliberately stricter than PEP-484" in note for note in getattr(e, "__notes__", []))), e
    ... else:
    ...    assert False
    """
    pass


def pytypes_multi_union(a: Union[list, tuple, None], b: list | tuple | None):
    """
    >>> pytypes_multi_union([1], [2])
    ('Python object', 'Python object')
    [[1], [2]]
    >>> pytypes_multi_union((1,), (2,))
    ('Python object', 'Python object')
    [(1,), (2,)]
    >>> pytypes_multi_union(1, 2)
    ('Python object', 'Python object')
    [1, 2]
    """
    print((typeof(a), typeof(b)))
    return [a, b]

_WARNINGS = """
15:32: Strings should no longer be used for type declarations. Use 'cython.int' etc. directly.
15:47: Dicts should no longer be used as type annotations. Use 'cython.int' etc. directly.
15:56: Strings should no longer be used for type declarations. Use 'cython.int' etc. directly.
15:77: Dicts should no longer be used as type annotations. Use 'cython.int' etc. directly.
15:85: Python type declaration in signature annotation does not refer to a Python type
15:85: Strings should no longer be used for type declarations. Use 'cython.int' etc. directly.
37:40: Found C type name 'long' in a Python annotation. Did you mean to use 'cython.long'?
37:40: Unknown type declaration 'long' in annotation, ignoring
37:66: PEP-484 recommends 'typing.Optional[...]' for arguments that can be None.
75:44: Found C type name 'long' in a Python annotation. Did you mean to use 'cython.long'?
75:44: Unknown type declaration 'long' in annotation, ignoring
75:70: PEP-484 recommends 'typing.Optional[...]' for arguments that can be None.
113:44: Found C type name 'long' in a Python annotation. Did you mean to use 'cython.long'?
113:44: Unknown type declaration 'long' in annotation, ignoring
113:70: PEP-484 recommends 'typing.Optional[...]' for arguments that can be None.
175:30: Tuples cannot be declared as simple tuples of types. Use 'tuple[type1, type2, ...]'.
175:59: Tuples cannot be declared as simple tuples of types. Use 'tuple[type1, type2, ...]'.
180:13: Tuples cannot be declared as simple tuples of types. Use 'tuple[type1, type2, ...]'.
315:44: Unknown type declaration in annotation, ignoring
346:15: Annotation ignored since class-level attributes must be Python objects. Were you trying to set up an instance attribute?
437:32: Unknown type declaration in annotation, ignoring
437:69: Unknown type declaration in annotation, ignoring
# DUPLICATE:
75:44: Found C type name 'long' in a Python annotation. Did you mean to use 'cython.long'?
75:44: Unknown type declaration 'long' in annotation, ignoring
# BUG:
75:0: 'pytypes_cpdef' redeclared
187:0: 'struct_io' redeclared
222:0: 'struct_convert' redeclared
241:0: 'exception_default' redeclared
272:0: 'exception_default_uint' redeclared
"""



def test1(t):
    """
    >>> test1( (1,2,3) )
    1
    """
    t,a,b = t
    return t

def test3(t):
    """
    >>> test3( (1,2,3) )
    3
    """
    a,b,t = t
    return t

def test(t):
    """
    >>> test( (1,2,3) )
    3
    """
    t,t,t = t
    return t

def testnonsense():
    """
    >>> testnonsense()     # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...
    """
    t,t,t = 1*2
    return t



# mode: error
# tag: warnings

cimport cython

@cython.dataclasses.dataclass
cdef class C:
    a: int
    # b: long  # only available in language_level=2, not worth testing
    c: Py_ssize_t
    d: float
    e: double


_WARNINGS = """
10:7: Found C type name 'Py_ssize_t' in a Python annotation. Did you mean to use 'cython.Py_ssize_t'?
10:7: Unknown type declaration 'Py_ssize_t' in annotation, ignoring
12:7: Found C type name 'double' in a Python annotation. Did you mean to use 'cython.double'?
12:7: Unknown type declaration 'double' in annotation, ignoring
"""



from cython.parallel import prange

def psum(int n):

    cdef int i
    cdef int sum = 0

    for i in prange(n, nogil=True, use_threads_if=n>1000):
        sum += i

    return sum

psum(30)        # Executed sequentially
psum(10000)     # Executed in parallel



# Run as:
#    python setup.py build_ext --inplace

import sys
sys.path.insert(0, "..")

from distutils.core import setup
from distutils.extension import Extension
from Cython.Build import cythonize

ext_modules = cythonize("**/*.pyx", exclude="numpy_*.pyx")

# Only compile the following if numpy is installed.
try:
    from numpy import get_include
except ImportError:
    pass
else:
    numpy_demo = [Extension("*",
                            ["numpy_*.pyx"],
                            include_dirs=get_include())]
    ext_modules.extend(cythonize(numpy_demo))

setup(
  name = 'Demos',
  ext_modules = ext_modules,
)



import numpy as np
import cython
# We now need to fix a datatype for our arrays. I've used the variable
# DTYPE for this, which is assigned to the usual NumPy runtime
# type info object.
DTYPE = np.intc

# @cython.cfunc means here that this function is a plain C function (so faster).
# To get all the benefits, we type the arguments and the return value.
@cython.exceptval(check=False)
@cython.cfunc
def clip(a: cython.int, min_value: cython.int, max_value: cython.int) -> cython.int:
    return min(max(a, min_value), max_value)


def compute(array_1, array_2, a: cython.int, b: cython.int, c: cython.int):

    # Annotation is also used within functions to type variables. It
    # can only be used at the top indentation level (there are non-trivial
    # problems with allowing them in other places, though we'd love to see
    # good and thought out proposals for it).
    x_max: cython.Py_ssize_t  = array_1.shape[0]
    y_max: cython.Py_ssize_t  = array_1.shape[1]

    assert array_1.shape == array_2.shape
    assert array_1.dtype == DTYPE
    assert array_2.dtype == DTYPE

    result = np.zeros((x_max, y_max), dtype=DTYPE)

    # It is very important to type ALL your variables. You do not get any
    # warnings if not, only much slower code (they are implicitly typed as
    # Python objects).
    # For the "tmp" variable, we want to use the same data type as is
    # stored in the array, so we use int because it correspond to np.intc.
    # NB! An important side-effect of this is that if "tmp" overflows its
    # datatype size, it will simply wrap around like in C, rather than raise
    # an error like in Python.

    tmp: cython.int

    # cython.Py_ssize_t is the proper C type for Python array indices.
    x: cython.Py_ssize_t
    y:  cython.Py_ssize_t

    for x in range(x_max):
        for y in range(y_max):

            tmp = clip(array_1[x, y], 2, 10)
            tmp = tmp * a + array_2[x, y] * b
            result[x, y] = tmp + c

    return result



# mode: compile

# cython: test_fail_if_c_code_has = __Pyx_ImportNumPyArrayTypeIfAvailable

ctypedef fused IntOrFloat:
    int
    float

# This function does not use buffers so has no reason to import numpy to
# look up dtypes. fused_buffers.pyx is the corresponding test for the case
# where numpy is imported
def f(IntOrFloat x):
    return x



 
from libc.stdlib cimport malloc, free

# Example C struct
ctypedef struct my_c_struct:
    int a
    int b



cdef class WrapperClass:
    """A wrapper class for a C/C++ data structure"""
    cdef my_c_struct *_ptr
    cdef bint ptr_owner

    def __cinit__(self):
        self.ptr_owner = False

    def __dealloc__(self):
        # De-allocate if not null and flag is set
        if self._ptr is not NULL and self.ptr_owner is True:
            free(self._ptr)
            self._ptr = NULL

    def __init__(self):
        # Prevent accidental instantiation from normal Python code
        # since we cannot pass a struct pointer into a Python constructor.
        raise TypeError("This class cannot be instantiated directly.")

    # Extension class properties
    @property
    def a(self):
        return self._ptr.a if self._ptr is not NULL else None

    @property
    def b(self):
        return self._ptr.b if self._ptr is not NULL else None


    @staticmethod
    cdef WrapperClass from_ptr(my_c_struct *_ptr, bint owner=False):
        """Factory function to create WrapperClass objects from
        given my_c_struct pointer.

        Setting ``owner`` flag to ``True`` causes
        the extension type to ``free`` the structure pointed to by ``_ptr``
        when the wrapper object is deallocated."""
        # Fast call to __new__() that bypasses the __init__() constructor.
        cdef WrapperClass wrapper = WrapperClass.__new__(WrapperClass)
        wrapper._ptr = _ptr
        wrapper.ptr_owner = owner
        return wrapper


    @staticmethod
    cdef WrapperClass new_struct():
        """Factory function to create WrapperClass objects with
        newly allocated my_c_struct"""
        cdef my_c_struct *_ptr = <my_c_struct *>malloc(sizeof(my_c_struct))

        if _ptr is NULL:
            raise MemoryError
        _ptr.a = 0
        _ptr.b = 0
        return WrapperClass.from_ptr(_ptr, owner=True)



from cython.cimports.c_func import c_call_returning_a_c_string

c_string = cython.declare(cython.p_char, c_call_returning_a_c_string())
if c_string is cython.NULL:
    ...  # handle error

py_string = cython.declare(bytes, c_string)




"""
>>> pxd_sqrt(9)
3.0
"""

import cython

if not cython.compiled:
    from math import sqrt as pxd_sqrt


@cython.test_assert_path_exists('//SimpleCallNode/NameNode[@type.is_pyobject = False]')
def call_pxd_sqrt(x):
    """
    >>> call_pxd_sqrt(9)
    3.0
    """
    return pxd_sqrt(x)



# -*- coding: utf-8 -*-

import sys

__doc__ = br"""
    >>> sa
    'abc'
    >>> ua
    'abc'
    >>> b
    '123'
    >>> c
    'S\xf8k ik'
    >>> d
    '\xfc\xd6\xe4'
    >>> e
    '\x03g\xf8\uf8d2S\xf8k ik'
    >>> f
    '\xf8'
    >>> g
    '\udc00'
    >>> h
    '\ud800'
    >>> q
    '\udc00\ud800'

    # The output of surrogate pairs differs between 16/32bit Unicode runtimes.
    #>>> p
    #u'\ud800\udc00'

    >>> add
    'S\xf8k ik\xfc\xd6\xe4abc'
    >>> null
    '\x00'
""".decode("ASCII") + b"""
    >>> len(sa)
    3
    >>> len(ua)
    3
    >>> len(b)
    3
    >>> len(c)
    6
    >>> len(d)
    3
    >>> len(e)
    10
    >>> len(f)
    1
    >>> len(g)
    1
    >>> len(h)
    1
    >>> len(q)
    2
    >>> len(q)
    2
    >>> len(add)
    12
    >>> len(null)
    1
    >>> sys.maxunicode >= 65535
    True
    >>> sys.maxunicode == 65535 and 1 or len(wide_literal) # test for wide build
    1
    >>> sys.maxunicode > 65535 and 2 or len(wide_literal)  # test for narrow build
    2
""".decode("ASCII") + u"""
    >>> ua ==  'abc'
    True
    >>> b ==  '123'
    True
    >>> c ==  'Søk ik'
    True
    >>> d ==  'üÖä'
    True
    >>> e ==  '\x03\x67\xf8\uf8d2Søk ik'     # unescaped by Cython
    True
    >>> e ==  '\\x03\\x67\\xf8\\uf8d2Søk ik' # unescaped by Python
    True
    >>> f ==  '\xf8'  # unescaped by Cython
    True
    >>> f ==  '\\xf8' # unescaped by Python
    True
    >>> g ==  '\\udc00' # unescaped by Python (required by doctest)
    True
    >>> h ==  '\\ud800' # unescaped by Python (required by doctest)
    True
    >>> p == (u'\\ud800\\udc00' if sys.maxunicode == 1114111 else  '\\U00010000')  or  p  # unescaped by Python (required by doctest)
    True
    >>> q ==  '\\udc00\\ud800'  or  q  # unescaped by Python (required by doctest)
    True
    >>> k ==  '\\N{SNOWMAN}' ==  '\\u2603'  or  k
    True
    >>> m ==  'abc\\\\xf8\\\\t\\u00f8\\U000000f8'  or  m  # unescaped by Python (required by doctest)
    True
    >>> add ==  'Søk ik' +  'üÖä' + 'abc'  or  add
    True
    >>> null ==  '\\x00' # unescaped by Python (required by doctest)
    True
    >>> wide_literal ==  '\\U00101234'   # unescaped by Python
    True
    >>> ustring_in_constant_tuple == ('a',  'abc',  '\\N{SNOWMAN}',  'x' * 3,  '\\N{SNOWMAN}' * 4 +  'O')  or  ustring_in_constant_tuple  # unescaped by Python
    True

    >>> expected =  '\U00101234'    # unescaped by Cython
    >>> if wide_literal == expected: print(True)
    ... else: print(repr(wide_literal), repr(expected), sys.maxunicode)
    True
"""

sa = 'abc'
ua = u'abc'

b = u'123'
c = u'Søk ik'
d = u'üÖä'
e = u'\x03\x67\xf8\uf8d2Søk ik'
f = u'\xf8'
g = u'\udc00'   # lone trail surrogate
h = u'\ud800'   # lone lead surrogate
k = u'\N{SNOWMAN}'
m = ur'abc\xf8\t\u00f8\U000000f8'
p = u'\ud800\udc00'  # surrogate pair
q = u'\udc00\ud800'  # reversed surrogate pair

add = u'Søk ik' + u'üÖä' + u'abc'
null = u'\x00'

wide_literal = u'\U00101234'

ustring_in_constant_tuple = ('a', u'abc', u'\N{SNOWMAN}', u'x' * 3, u'\N{SNOWMAN}' * 4 + u'O')



# mode: compile

from spam import *
from ...spam.foo import *
from ... spam.foo import *
from .. . spam.foo import *
from . . . spam.foo import *
from . .. spam.foo import *
from . import *
from ... import *
from .. . import *
from . .. import *
from . . . import *



# mode: run
# tag: cpp, werror, cpp11

from cython.operator cimport dereference as deref

from libcpp cimport bool
from libcpp.algorithm cimport merge, inplace_merge
from libcpp.vector cimport vector


cdef bool less(int a, int b):
    return a < b

def test_merge(vector[int] v1, vector[int] v2):
    """
    Test merge.

    >>> test_merge([1, 3, 5], [2, 4])
    [1, 2, 3, 4, 5]
    """
    cdef vector[int] out = vector[int](v1.size() + v2.size())
    merge(v1.begin(), v1.end(), v2.begin(), v2.end(), out.begin())
    return out

def test_merge_with_bin_pred(vector[int] v1, vector[int] v2):
    """
    Test merge with binary predicate

    >>> test_merge_with_bin_pred([1, 3, 5], [2, 4])
    [1, 2, 3, 4, 5]
    """
    cdef vector[int] out = vector[int](v1.size() + v2.size())
    merge(v1.begin(), v1.end(), v2.begin(), v2.end(), out.begin(), less)
    return out

def test_inplace_merge(vector[int] v):
    """
    Test inplace_merge.

    >>> test_inplace_merge([4, 5, 6, 1, 2, 3])
    [1, 2, 3, 4, 5, 6]
    """
    inplace_merge(v.begin(), v.begin() + 3, v.end())
    return v

def test_inplace_merge_with_bin_pred(vector[int] v):
    """
    Test inplace_merge with binary predicate

    >>> test_inplace_merge_with_bin_pred([4, 5, 6, 1, 2, 3])
    [1, 2, 3, 4, 5, 6]
    """
    inplace_merge(v.begin(), v.begin() + 3, v.end(), less)
    return v


# mode: run
# tag: cpp, werror, no-cpp-locals, cpp11

from cython.operator cimport dereference as d
from cython.operator cimport preincrement as incr

from libcpp.vector cimport vector

ctypedef vector[int] ivector

def test_vector_data():
    """
    >>> test_vector_data()
    (77, 77)
    """
    cdef:
        int* data
        const int* const_data
    int_vector = ivector()
    int_vector.push_back(77)
    data = int_vector.data()
    const_data = int_vector.const_data()
    return data[0], const_data[0]

def test_vector_emplace():
    """
    >>> test_vector_emplace()
    [0, 7]
    """
    cdef ivector v
    v.emplace(v.cend())
    v.emplace(v.cend(), 7)
    return v

def test_vector_emplace_back():
    """
    >>> test_vector_emplace_back()
    [0, 7]
    """
    cdef ivector v
    v.emplace_back()
    v.emplace_back(7)
    return v

def test_vector_shrink_to_fit():
    """
    >>> test_vector_shrink_to_fit()
    """
    cdef ivector v
    v.shrink_to_fit()

def const_iteration_test(L):
    """
    >>> const_iteration_test([1,2,4,8])
    1
    2
    4
    8
    """
    v = new vector[int]()
    try:
        for a in L:
            v.push_back(a)
        it = v.cbegin()
        while it != v.cend():
            a = d(it)
            incr(it)
            print(a)
    finally:
        del v



# distutils: language = c++

from cython.cimports.cpython import Py_buffer
from cython.cimports.libcpp.vector import vector

@cython.cclass
class Matrix:

    view_count: cython.int

    ncols: cython.Py_ssize_t
    v: vector[cython.float]
    # ...

    def __cinit__(self, ncols: cython.Py_ssize_t):
        self.ncols = ncols
        self.view_count = 0

    def add_row(self):
        if self.view_count > 0:
            raise ValueError("can't add row while being viewed")
        self.v.resize(self.v.size() + self.ncols)

    def __getbuffer__(self, buffer: cython.pointer[Py_buffer], flags: cython.int):
        # ... as before

        self.view_count += 1

    def __releasebuffer__(self, buffer: cython.pointer[Py_buffer]):
        self.view_count -= 1



# distutils: language = c++

from cython.parallel import parallel, prange
from cython.cimports.libc.stdlib import malloc, free
from cython.cimports.libcpp.algorithm import nth_element
import cython
from cython.operator import dereference

import numpy as np

@cython.boundscheck(False)
@cython.wraparound(False)
def median_along_axis0(x: cython.double[:,:]):
    out: cython.double[::1] = np.empty(x.shape[1])
    i: cython.Py_ssize_t
    j: cython.Py_ssize_t
    scratch: cython.p_double
    median_it: cython.p_double
    with cython.nogil, parallel():
        # allocate scratch space per loop
        scratch = cython.cast(
            cython.p_double,
            malloc(cython.sizeof(cython.double)*x.shape[0]))
        try:
            for i in prange(x.shape[1]):
                # copy row into scratch space
                for j in range(x.shape[0]):
                    scratch[j] = x[j, i]
                median_it = scratch + x.shape[0]//2
                nth_element(scratch, median_it, scratch + x.shape[0])
                # for the sake of a simple example, don't handle even lengths...
                out[i] = dereference(median_it)
        finally:
            free(scratch)
    return np.asarray(out)



# mode: run
# tag: pyclass, global


pyvar = 2

class TestPyAttr(object):
    """
    >>> TestPyAttr.pyvar    # doctest: +ELLIPSIS
    Traceback (most recent call last):
    AttributeError: ...TestPyAttr...has no attribute 'pyvar'...
    >>> TestPyAttr.pyval1
    3
    >>> TestPyAttr.pyval2
    2
    """
    pyvar = 3
    pyval1 = pyvar
    del pyvar
    pyval2 = pyvar


import cython
cdefvar = cython.declare(int, 10)

class TestCdefAttr(object):
    """
    >>> TestCdefAttr.cdefvar   # doctest: +ELLIPSIS
    Traceback (most recent call last):
    AttributeError: ...TestCdefAttr...has no attribute 'cdefvar'...
    >>> TestCdefAttr.cdefval1
    11

    >>> #TestCdefAttr.cdefval2
    """
    cdefvar = 11
    cdefval1 = cdefvar
    del cdefvar
    # cdefval2 = cdefvar       # FIXME: doesn't currently work ...


class ForLoopInPyClass(object):
    """
    >>> ForLoopInPyClass.i    # doctest: +ELLIPSIS
    Traceback (most recent call last):
    AttributeError: ...ForLoopInPyClass... has no attribute ...i...
    >>> ForLoopInPyClass.k
    0
    >>> ForLoopInPyClass.m
    1
    """
    for i in range(0):
        pass

    for k in range(1):
        pass

    for m in range(2):
        pass


def del_in_class(x):
    """
    >>> del_in_class(True)
    no error
    >>> del_in_class(False)
    NameError
    """
    try:
        class Test(object):
            if x:
                attr = 1
            del attr
    except NameError:
        print("NameError")
    else:
        print("no error")



# ticket: t491

def test_genexpr():
    """
    >>> gen = test_genexpr()
    >>> list(gen)
    [0, 1, 2, 3, 4]
    """
    return (i for i in range(5))

def test_genexpr_typed():
    """
    >>> gen = test_genexpr_typed()
    >>> list(gen)
    [0, 1, 2, 3, 4]
    """
    cdef int i
    return (i for i in range(5))

def test_genexpr_funccall():
    """
    >>> test_genexpr_funccall()
    [0, 1, 2, 3, 4]
    """
    return list(i for i in range(5))

def test_genexpr_scope():
    """
    >>> test_genexpr_scope()
    ([0, 1, 2, 3, 4], 'abc')
    """
    i = 'abc'
    gen = (i for i in range(5))
    lst = list(gen)
    return lst, i

def test_genexpr_closure():
    """
    >>> gen = test_genexpr_closure()
    >>> list(gen)
    ['a', 'b', 'c']
    """
    abc = 'a' + 'b' + 'c'
    return (c for c in abc)



"""
Dump the GitHub issues of the current project to a file (.json.gz).

Usage:  python3 Tools/dump_github_issues.py
"""

import configparser
import gzip
import json
import os.path

from datetime import datetime
from urllib.request import urlopen

GIT_CONFIG_FILE = ".git/config"


class RateLimitReached(Exception):
    pass


def gen_urls(repo):
    i = 0
    while True:
        yield f"https://api.github.com/repos/{repo}/issues?state=all&per_page=100&page={i}"
        i += 1


def read_rate_limit():
    with urlopen("https://api.github.com/rate_limit") as p:
        return json.load(p)


def parse_rate_limit(limits):
    limits = limits['resources']['core']
    return limits['limit'], limits['remaining'], datetime.fromtimestamp(limits['reset'])


def load_url(url):
    with urlopen(url) as p:
        data = json.load(p)
    if isinstance(data, dict) and 'rate limit' in data.get('message', ''):
        raise RateLimitReached()

    assert isinstance(data, list), type(data)
    return data or None  # None indicates empty last page


def join_list_data(lists):
    result = []
    for data in lists:
        if not data:
            break
        result.extend(data)
    return result


def output_filename(repo):
    timestamp = datetime.now()
    return f"github_issues_{repo.replace('/', '_')}_{timestamp.strftime('%Y%m%d_%H%M%S')}.json.gz"


def write_gzjson(file_name, data, indent=2):
    with gzip.open(file_name, "wt", encoding='utf-8') as gz:
        json.dump(data, gz, indent=indent)


def find_origin_url(git_config=GIT_CONFIG_FILE):
    assert os.path.exists(git_config)
    parser = configparser.ConfigParser()
    parser.read(git_config)
    return parser.get('remote "origin"', 'url')


def parse_repo_name(git_url):
    if git_url.endswith('.git'):
        git_url = git_url[:-4]
    return '/'.join(git_url.split('/')[-2:])


def dump_issues(repo):
    """Main entry point."""
    print(f"Reading issues from repo '{repo}'")
    urls = gen_urls(repo)
    try:
        paged_data = map(load_url, urls)
        issues = join_list_data(paged_data)
    except RateLimitReached:
        limit, remaining, reset_time = parse_rate_limit(read_rate_limit())
        print(f"FAILURE: Rate limits ({limit}) reached, remaining: {remaining}, reset at {reset_time}")
        return

    filename = output_filename(repo)
    print(f"Writing {len(issues)} to {filename}")
    write_gzjson(filename, issues)


### TESTS

def test_join_list_data():
    assert join_list_data([]) == []
    assert join_list_data([[1,2]]) == [1,2]
    assert join_list_data([[1,2], [3]]) == [1,2,3]
    assert join_list_data([[0], [1,2], [3]]) == [0,1,2,3]
    assert join_list_data([[0], [1,2], [[[]],[]]]) == [0,1,2,[[]],[]]


def test_output_filename():
    filename = output_filename("re/po")
    import re
    assert re.match(r"github_issues_re_po_[0-9]{8}_[0-9]{6}\.json", filename)


def test_find_origin_url():
    assert find_origin_url()


def test_parse_repo_name():
    assert parse_repo_name("https://github.com/cython/cython") == "cython/cython"
    assert parse_repo_name("git+ssh://git@github.com/cython/cython.git") == "cython/cython"
    assert parse_repo_name("git+ssh://git@github.com/fork/cython.git") == "fork/cython"


def test_write_gzjson():
    import tempfile
    with tempfile.NamedTemporaryFile() as tmp:
        write_gzjson(tmp.name, [{}])

        # test JSON format
        with gzip.open(tmp.name) as f:
            assert json.load(f) == [{}]

        # test indentation
        with gzip.open(tmp.name) as f:
            assert f.read() == b'[\n  {}\n]'


### MAIN

if __name__ == '__main__':
    repo_name = parse_repo_name(find_origin_url())
    dump_issues(repo_name)



# mode: error

# invalid syntax (as handled by the parser)

def syntax():
    *a, *b = 1,2,3,4,5


_ERRORS = u"""
6:4: more than 1 starred expression in assignment
6:8: more than 1 starred expression in assignment
"""



cimport cython
cimport check_fused_types_pxd

import math

ctypedef char *string_t

fused_t = cython.fused_type(int, long, float, string_t)
other_t = cython.fused_type(int, long)
base_t = cython.fused_type(short, int)

# complex_t = cython.fused_type(cython.floatcomplex, cython.doublecomplex)
cdef fused complex_t:
    float complex
    double complex

ctypedef base_t **base_t_p_p

# ctypedef cython.fused_type(char, base_t_p_p, fused_t, complex_t) composed_t
cdef fused composed_t:
    char
    int
    float
    string_t
    cython.pp_int
    float complex
    double complex
    int complex
    long complex


cdef func(fused_t a, other_t b):
    cdef int int_a
    cdef string_t string_a
    cdef other_t other_a

    if fused_t is other_t:
        print 'fused_t is other_t'
        other_a = a

    if fused_t is int:
        print 'fused_t is int'
        int_a = a

    if fused_t is string_t:
        print 'fused_t is string_t'
        string_a = a

    if fused_t in check_fused_types_pxd.unresolved_t:
        print 'fused_t in unresolved_t'

    if int in check_fused_types_pxd.unresolved_t:
        print 'int in unresolved_t'

    if string_t in check_fused_types_pxd.unresolved_t:
        print 'string_t in unresolved_t'


def test_int_int():
    """
    >>> test_int_int()
    fused_t is other_t
    fused_t is int
    fused_t in unresolved_t
    int in unresolved_t
    """
    cdef int x = 1
    cdef int y = 2

    func(x, y)

def test_int_long():
    """
    >>> test_int_long()
    fused_t is int
    fused_t in unresolved_t
    int in unresolved_t
    """
    cdef int x = 1
    cdef long y = 2

    func(x, y)

def test_float_int():
    """
    >>> test_float_int()
    fused_t in unresolved_t
    int in unresolved_t
    """
    cdef float x = 1
    cdef int y = 2

    func(x, y)

def test_string_int():
    """
    >>> test_string_int()
    fused_t is string_t
    int in unresolved_t
    """
    cdef string_t x = b"spam"
    cdef int y = 2

    func(x, y)


cdef if_then_else(fused_t a, other_t b):
    cdef other_t other_a
    cdef string_t string_a
    cdef fused_t specific_a

    if fused_t is other_t:
        print 'fused_t is other_t'
        other_a = a
    elif fused_t is string_t:
        print 'fused_t is string_t'
        string_a = a
    else:
        print 'none of the above'
        specific_a = a

def test_if_then_else_long_long():
    """
    >>> test_if_then_else_long_long()
    fused_t is other_t
    """
    cdef long x = 0, y = 0
    if_then_else(x, y)

def test_if_then_else_string_int():
    """
    >>> test_if_then_else_string_int()
    fused_t is string_t
    """
    cdef string_t x = b"spam"
    cdef int y = 0
    if_then_else(x, y)

def test_if_then_else_float_int():
    """
    >>> test_if_then_else_float_int()
    none of the above
    """
    cdef float x = 0.0
    cdef int y = 1
    if_then_else(x, y)


cdef composed_t composed(composed_t x, composed_t y):
    if composed_t in base_t_p_p or composed_t is string_t:
        if string_t == composed_t:
            print x.decode('ascii'), y.decode('ascii')
        else:
            print x[0][0], y[0][0]

        return x
    elif composed_t == string_t:
        print 'this is never executed'
    elif list():
        print 'neither is this one'
    else:
        if composed_t not in complex_t:
            print 'not a complex number'
            print <int> x, <int> y
        else:
            print 'it is a complex number'
            print x.real, x.imag

        return x + y

def test_composed_types():
    """
    >>> test_composed_types()
    it is a complex number
    0.5 0.6
    9 4
    <BLANKLINE>
    not a complex number
    7 8
    15
    <BLANKLINE>
    7 8
    <BLANKLINE>
    spam eggs
    spam
    """
    cdef double complex a = 0.5 + 0.6j, b = 0.4 -0.2j, result
    cdef int c = 7, d = 8
    cdef int *cp = &c, *dp = &d
    cdef string_t e = "spam", f = "eggs"

    result = composed(a, b)
    print int(math.ceil(result.real * 10)), int(math.ceil(result.imag * 10))
    print

    print composed(c, d)
    print

    composed(&cp, &dp)
    print

    print composed(e, f).decode('ascii')




import cython

@cython.freelist(8)
@cython.cclass
class Penguin:
    food: object
    def __cinit__(self, food):
        self.food = food

penguin = Penguin('fish 1')
penguin = None
penguin = Penguin('fish 2')  # does not need to allocate memory!



cdef class Spam:
    cdef public int tons
    cdef readonly float tastiness
    cdef int temperature

    def __init__(self, tons, tastiness, temperature):
        self.tons = tons
        self.tastiness = tastiness
        self.temperature = temperature

    def get_temperature(self):
        return self.temperature



cdef extern from *:
    """
    #if defined(_WIN32) || defined(MS_WINDOWS) || defined(_MSC_VER)
      #include "stdlib.h"
      #define myapp_sleep(m)  _sleep(m)
    #else
      #include <unistd.h>
      #define myapp_sleep(m)  ((void) usleep((m) * 1000))
    #endif
    """
    # using "myapp_" prefix in the C code to prevent C naming conflicts
    void msleep "myapp_sleep"(int milliseconds) nogil

msleep(milliseconds=1)



# mode: compile
# tag: cpp,cpp11


cpdef enum class Spam:
    a, b
    c
    d
    e
    f = 42


cpdef enum class Cheese(unsigned int):
    x = 1
    y = 2


cdef enum struct parrot_state:
    alive = 1
    dead = 0


cdef void eggs():
    cdef Spam s1
    s1 = Spam.a
    s2 = Spam.b

    cdef Cheese c1
    c1 = Cheese.x

eggs()


# enum interdependency
cdef enum class Color(int):
    RED = 1
    GREEN = 2


cdef enum class Color2(int):
    RED = (<int> Color.RED)
    GREEN = (<int> Color.GREEN)


# enum class as cdef class function parameter
cdef class A:
    cdef Spam f(self, Spam s):
        return s



# cython: language_level=3

from libcpp.string cimport string

cdef extern from "cpp_nested_names_helper.h":
    cdef cppclass Outer:
        cppclass Nested:
            cppclass NestedNested:
                string get_str()

            string get_str()

            @staticmethod
            NestedNested get()

        @staticmethod
        Nested get()



# mode: error
# tag: pxd

# OK

cdef int wider_exception_check(int x, int y) except 0:
    return 2

cdef int no_exception_raised(int x, int y):
    return 2

cdef int any_exception_value1(int x, int y) except *:
    return 2

cdef int any_exception_value2(int x, int y) except -1:
    return 2

cdef int any_exception_value3(int x, int y) except -2:
    return 2

cdef int any_exception_value4(int x, int y) except? -2:
    return 2

cdef int optimised_exception_value(int x, int y) except *:  # => except? -1
    return 2


# NOK

cdef int wrong_args(int x, int y):
    return 2

cdef int wrong_return_type(int x, int y):
    return 2

cdef int foreign_exception_value(int x, int y):
    return 2

cdef int narrower_exception_check(int x, int y) except? 0:
    return 2

cdef int wrong_exception_value(int x, int y) except 1:
    return 2

cdef int wrong_exception_value_check(int x, int y) except? 1:
    return 2

cdef int wrong_exception_value_optimised_check(int x, int y) except *:
    return 2

cdef int wrong_exception_value_optimised(int x, int y) except *:
    return 2

cdef int narrower_exception_check_optimised(int x, int y) except *:
    return 2


_ERRORS = """
30:0: Function signature does not match previous declaration
33:0: Function signature does not match previous declaration
36:0: Function signature does not match previous declaration
39:0: Function signature does not match previous declaration
42:0: Function signature does not match previous declaration
45:0: Function signature does not match previous declaration
48:0: Function signature does not match previous declaration
51:0: Function signature does not match previous declaration
54:0: Function signature does not match previous declaration
"""



# ticket: 2891
# tag: c, no-cpp

cdef extern from "complex_numbers_c99_T398.h": pass

from libc.complex cimport cimag, creal, cabs, carg


def test_decomposing(double complex z):
    """
    >>> test_decomposing(3+4j)
    (3.0, 4.0, 5.0, 0.9272952180016122)
    """

    return (creal(z), cimag(z), cabs(z), carg(z))



cimport cython

cdef sorteditems(d):
    return tuple(sorted(d.items()))


cdef class Silly:

    def __init__(self, *a):
        """
        >>> s = Silly(1,2,3, 'test')
        """

    def spam(self, x, y, z):
        """
        >>> s = Silly()
        >>> s.spam(1,2,3)
        (1, 2, 3)
        >>> s.spam(1,2)
        Traceback (most recent call last):
        TypeError: spam() takes exactly 3 positional arguments (2 given)
        >>> s.spam(1,2,3,4)
        Traceback (most recent call last):
        TypeError: spam() takes exactly 3 positional arguments (4 given)
        >>> s.spam(1,2,3, a=1)
        Traceback (most recent call last):
        TypeError: spam() got an unexpected keyword argument 'a'
        """
        return (x, y, z)

    def grail(self, x, y, z, *a):
        """
        >>> s = Silly()
        >>> s.grail(1,2,3)
        (1, 2, 3, ())
        >>> s.grail(1,2,3,4)
        (1, 2, 3, (4,))
        >>> s.grail(1,2,3,4,5,6,7,8,9)
        (1, 2, 3, (4, 5, 6, 7, 8, 9))
        >>> s.grail(1,2)
        Traceback (most recent call last):
        TypeError: grail() takes at least 3 positional arguments (2 given)
        >>> s.grail(1,2,3, a=1)
        Traceback (most recent call last):
        TypeError: grail() got an unexpected keyword argument 'a'
        """
        return (x, y, z, a)

    def swallow(self, x, y, z, **k):
        """
        >>> s = Silly()
        >>> s.swallow(1,2,3)
        (1, 2, 3, ())
        >>> s.swallow(1,2,3,4)
        Traceback (most recent call last):
        TypeError: swallow() takes exactly 3 positional arguments (4 given)
        >>> s.swallow(1,2,3, a=1, b=2)
        (1, 2, 3, (('a', 1), ('b', 2)))
        >>> s.swallow(1,2,3, x=1)
        Traceback (most recent call last):
        TypeError: swallow() got multiple values for keyword argument 'x'
        """
        return (x, y, z, sorteditems(k))

    def creosote(self, x, y, z, *a, **k):
        """
        >>> s = Silly()
        >>> s.creosote(1,2,3)
        (1, 2, 3, (), ())
        >>> s.creosote(1,2,3,4)
        (1, 2, 3, (4,), ())
        >>> s.creosote(1,2,3, a=1)
        (1, 2, 3, (), (('a', 1),))
        >>> s.creosote(1,2,3,4, a=1, b=2)
        (1, 2, 3, (4,), (('a', 1), ('b', 2)))
        >>> s.creosote(1,2,3,4, x=1)
        Traceback (most recent call last):
        TypeError: creosote() got multiple values for keyword argument 'x'
        """
        return (x, y, z, a, sorteditems(k))

    def onlyt(self, *a):
        """
        >>> s = Silly()
        >>> s.onlyt(1)
        (1,)
        >>> s.onlyt(1,2)
        (1, 2)
        >>> s.onlyt(a=1)
        Traceback (most recent call last):
        TypeError: onlyt() got an unexpected keyword argument 'a'
        >>> s.onlyt(1, a=2)
        Traceback (most recent call last):
        TypeError: onlyt() got an unexpected keyword argument 'a'
        """
        return a

    @cython.binding(False)  # passthrough of exact same tuple can't work with binding
    def onlyt_nobinding(self, *a):
        """
        >>> s = Silly()
        >>> s.onlyt_nobinding(1)
        (1,)
        >>> s.onlyt_nobinding(1,2)
        (1, 2)
        >>> s.onlyt_nobinding(a=1)
        Traceback (most recent call last):
        TypeError: onlyt_nobinding() got an unexpected keyword argument 'a'
        >>> s.onlyt_nobinding(1, a=2)
        Traceback (most recent call last):
        TypeError: onlyt_nobinding() got an unexpected keyword argument 'a'
        >>> test_no_copy_args(s.onlyt_nobinding)
        True
        """
        return a

    def onlyk(self, **k):
        """
        >>> s = Silly()
        >>> s.onlyk(a=1)
        (('a', 1),)
        >>> s.onlyk(a=1, b=2)
        (('a', 1), ('b', 2))
        >>> s.onlyk(1)
        Traceback (most recent call last):
        TypeError: onlyk() takes exactly 0 positional arguments (1 given)
        >>> s.onlyk(1, 2)
        Traceback (most recent call last):
        TypeError: onlyk() takes exactly 0 positional arguments (2 given)
        >>> s.onlyk(1, a=1, b=2)
        Traceback (most recent call last):
        TypeError: onlyk() takes exactly 0 positional arguments (1 given)
        """
        return sorteditems(k)

    def tk(self, *a, **k):
        """
        >>> s = Silly()
        >>> s.tk(a=1)
        (('a', 1),)
        >>> s.tk(a=1, b=2)
        (('a', 1), ('b', 2))
        >>> s.tk(1)
        (1,)
        >>> s.tk(1, 2)
        (1, 2)
        >>> s.tk(1, a=1, b=2)
        (1, ('a', 1), ('b', 2))
        """
        return a + sorteditems(k)

    @cython.binding(False)  # passthrough of exact same tuple can't work with binding
    def t_kwonly(self, *a, k):
        """
        >>> s = Silly()
        >>> test_no_copy_args(s.t_kwonly, k=None)
        True
        """
        return a


def test_no_copy_args(func, **kw):
    """
    func is a function such that func(*args, **kw) returns args.
    We test that no copy is made of the args tuple.
    This tests both the caller side and the callee side.
    """
    args = (1, 2, 3)
    return func(*args, **kw) is args



# mode: run
# cython:infer_types=True

cimport cython

from libc.stdint cimport int64_t


### Tests for external typedefs.

@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_td_int64_t']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_typedef(a: int64_t, b: cython.int):
    """
    >>> divmod_typedef(10, 5)
    (2, 0)
    >>> divmod_typedef(9191, 4096)
    (2, 999)
    >>> divmod_typedef(-420000000000, 1000)
    (-420000000, 0)

    >>> divmod_typedef(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_typedef(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_td_int64_t']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_typedef_const(a: int64_t):
    """
    >>> divmod_typedef_const(10)
    (0, 10)
    >>> divmod_typedef_const(9191)
    (9, 191)
    >>> divmod_typedef_const(-420000000000)
    (-420000000, 0)
    """
    result = divmod(a, 1000)
    return result


### Tests for internal and mixed typedefs.

ctypedef long my_long_type
ctypedef int64_t my_int64_type


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_td_int64_t']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_typedef_mixed(a: my_int64_type, b: cython.int):
    """
    >>> divmod_typedef_mixed(10, 5)
    (2, 0)
    >>> divmod_typedef_mixed(9191, 4096)
    (2, 999)
    >>> divmod_typedef_mixed(-420000000000, 1000)
    (-420000000, 0)

    >>> divmod_typedef_mixed(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_typedef_mixed(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result


@cython.test_assert_path_exists(
    "//SimpleCallNode//NameNode[@entry.name = 'divmod']",
    "//SimpleCallNode//NameNode[@entry.cname = '__Pyx_divmod_int_long']",
    "//ReturnStatNode//CoerceToPyTypeNode",
)
def divmod_typedef_internal(a: my_long_type, b: cython.int):
    """
    >>> divmod_typedef_internal(10, 5)
    (2, 0)
    >>> divmod_typedef_internal(9191, 4096)
    (2, 999)

    >>> divmod_typedef_internal(33, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    >>> divmod_typedef_internal(0, 0)  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ZeroDivisionError: ...
    """
    result = divmod(a, b)
    return result



# cython: warn.maybe_uninitialized=True
# mode: error
# tag: werror

def exc_target():
    try:
        {}['foo']
    except KeyError, e:
        pass
    except IndexError, i:
        pass
    return e, i

def exc_body():
    try:
        a = 1
    except Exception:
        pass
    return a

def exc_else_pos():
    try:
        pass
    except Exception, e:
        pass
    else:
        e = 1
    return e

def exc_body_pos(d):
    try:
        a = d['foo']
    except KeyError:
        a = None
    return a

def exc_pos():
    try:
        a = 1
    except Exception:
        a = 1
    return a

def exc_finally():
    try:
        a = 1
    finally:
        pass
    return a

def exc_finally2():
    try:
        pass
    finally:
        a = 1
    return a


def exc_assmt_except(a):
    try:
        x = a
    except:
        return x

def exc_assmt_finaly(a):
    try:
        x = a
    except:
        return x

def raise_stat(a):
    try:
        if a < 0:
            raise IndexError
    except IndexError:
        oops = 1
    print oops

def try_loop(args):
    try:
        x = 0
        for i in args:
            if i is 0:
                continue
            elif i is None:
                break
            elif i is False:
                return
            i()
    except ValueError:
        x = 1
    finally:
        return x

def try_finally(a):
    try:
        for i in a:
            if i > 0:
                x = 1
    finally:
        return x

def try_finally_nested(m):
    try:
        try:
            try:
                f = m()
            except:
                pass
        finally:
            pass
    except:
        print f

_ERRORS = """
12:11: local variable 'e' might be referenced before assignment
12:14: local variable 'i' might be referenced before assignment
19:11: local variable 'a' might be referenced before assignment
63:15: local variable 'x' might be referenced before assignment
69:15: local variable 'x' might be referenced before assignment
77:10: local variable 'oops' might be referenced before assignment
93:15: local variable 'x' might be referenced before assignment
101:15: local variable 'x' might be referenced before assignment
113:14: local variable 'f' might be referenced before assignment
"""



# mode: run
# tag: cpp, cpp11

# Do not add to this test - it's to ensure that all the right utility code
# is generated for exception_ptr_error_handler *if we don't do any other C++ error handling*.

from libcpp.exception cimport exception_ptr_error_handler

cdef extern from *:
    """
    void thrower() {
        throw std::runtime_error("bad");
    }
    """
    void thrower() except +exception_ptr_error_handler

def test():
    """
    >>> test()  # doctest: +ELLIPSIS
    RuntimeError(<capsule object "std::exception_ptr wrapper" at ...)
    """
    try:
        thrower()
    except Exception as e:
        return e



cdef class A:
    """
    >>> A.__init__.__doc__
    'A.__init__ docstring'
    >>> A.__len__.__doc__
    'A.__len__ docstring'
    >>> A.__add__.__doc__
    'A.__add__ docstring'
    >>> A.__getattr__.__doc__
    'A.__getattr__ docstring'
    """
    def __init__(self):
        "A.__init__ docstring"
    def __len__(self):
        "A.__len__ docstring"
    def __add__(self, other):
        "A.__add__ docstring"
    def __getattr__(self, name):
        "A.__getattr__ docstring"

cdef class B(A):
    """
    >>> B.__init__.__doc__
    'A.__init__ docstring'
    >>> B.__len__.__doc__
    'B.__len__ docstring'
    >>> B.__add__.__doc__
    'A.__add__ docstring'
    >>> B.__getattr__.__doc__
    'A.__getattr__ docstring'
    """
    def __len__(self):
        "B.__len__ docstring"

class C(A):
    """
    >>> C.__init__.__doc__
    'A.__init__ docstring'
    >>> C.__len__.__doc__
    'C.__len__ docstring'
    >>> C.__add__.__doc__
    'A.__add__ docstring'
    >>> C.__getattr__.__doc__
    'A.__getattr__ docstring'
    """
    def __len__(self):
        "C.__len__ docstring"



from __future__ import print_function

from cython cimport typeof

ctypedef bint mybool

cdef mybool mybul = True
cdef bint bul = True
cdef int num = 42


def CondExprNode_to_obj(test):
    """
    >>> CondExprNode_to_obj(True)
    Python object | Python object
    2
    >>> CondExprNode_to_obj(False)
    Python object | Python object
    84
    """

    print(typeof(mybul if test else num), "|", typeof(bul if test else num))

    return (mybul if test else num) + (bul if test else num)


def BoolBinopNode_to_obj():
    """
    >>> BoolBinopNode_to_obj()
    Python object | Python object
    2
    """

    print(typeof(mybul or num), "|", typeof(bul or num))

    return (mybul or num) + (bul or num)


cdef int test_bool(mybool arg):
    return <int>arg


def CondExprNode_to_bool(test):
    """
    >>> CondExprNode_to_bool(True)
    bint | bint
    0
    >>> CondExprNode_to_bool(False)
    bint | bint
    2
    """

    print(typeof(not mybul if test else mybul), "|", typeof(not bul if test else bul))

    # test_bool() would silently crash if one of the types is cast
    # to Python object and not just assigned.
    # It happens when a type is wrongly inferred as Python object
    # instead of bint or mybool.
    return test_bool(not mybul if test else mybul) + test_bool(not bul if test else bul)


def BoolBinopNode_to_bool():
    """
    >>> BoolBinopNode_to_bool()
    bint | bint
    2
    """

    print(typeof(not mybul or mybul), "|", typeof(not bul or bul))

    return test_bool(not mybul or mybul) + test_bool(not bul or bul)



# ticket: t544

def count(i=[0]):
    i[0] += 1
    return i[0]

def test(x):
    """
    >>> def py_count(i=[0]):
    ...     i[0] += 1
    ...     return i[0]
    >>> 1 in (py_count(), py_count(), py_count(), py_count())
    True
    >>> 4 in (py_count(), py_count(), py_count(), py_count())
    False
    >>> 12 in (py_count(), py_count(), py_count(), py_count())
    True

    >>> test(1)
    True
    >>> test(4)
    False
    >>> test(12)
    True
    """
    return x in (count(), count(), count(), count())



# mode: run

# __getattribute__ and __getattr__ special methods for a single class.


cdef class just_getattribute:
    """
    >>> a = just_getattribute()
    >>> a.called
    1
    >>> a.called
    2
    >>> a.bar
    'bar'
    >>> a.called
    4
    >>> try: a.invalid
    ... except AttributeError: pass
    ... else: print("NOT RAISED!")
    >>> a.called
    6
    """
    cdef readonly int called
    def __getattribute__(self,n):
        self.called += 1
        if n == 'bar':
            return n
        elif n == 'called':
            return self.called
        else:
            raise AttributeError


cdef class just_getattr:
    """
    >>> a = just_getattr()
    >>> a.called
    0
    >>> a.called
    0
    >>> a.foo
    10
    >>> a.called
    0
    >>> a.bar
    'bar'
    >>> a.called
    1
    >>> try: a.invalid
    ... except AttributeError: pass
    ... else: print("NOT RAISED!")
    >>> a.called
    2
    """
    cdef readonly int called
    cdef readonly int foo
    def __init__(self):
        self.foo = 10
    def __getattr__(self,n):
        self.called += 1
        if n == 'bar':
            return n
        else:
            raise AttributeError


cdef class both:
    """
    >>> a = both()
    >>> (a.called_getattr, a.called_getattribute)
    (0, 2)
    >>> a.foo
    10
    >>> (a.called_getattr, a.called_getattribute)
    (0, 5)
    >>> a.bar
    'bar'
    >>> (a.called_getattr, a.called_getattribute)
    (1, 8)
    >>> try: a.invalid
    ... except AttributeError: pass
    ... else: print("NOT RAISED!")
    >>> (a.called_getattr, a.called_getattribute)
    (2, 11)
    """
    cdef readonly int called_getattribute
    cdef readonly int called_getattr
    cdef readonly int foo
    def __init__(self):
        self.foo = 10

    def __getattribute__(self,n):
        self.called_getattribute += 1
        if n == 'foo':
            return self.foo
        elif n == 'called_getattribute':
            return self.called_getattribute
        elif n == 'called_getattr':
            return self.called_getattr
        else:
            raise AttributeError

    def __getattr__(self,n):
        self.called_getattr += 1
        if n == 'bar':
            return n
        else:
            raise AttributeError



#!/usr/bin/python
# micro benchmarks for generators

COUNT = 20_000

import cython
import time


def count_to(N: cython.Py_ssize_t):
    i: cython.Py_ssize_t
    for i in range(N):
        yield i

def round_robin(*_iterators):
    i: cython.Py_ssize_t

    iterators = list(_iterators)
    to_drop = []

    while iterators:
        for i, it in enumerate(iterators):
            try:
                value = next(it)
            except StopIteration:
                to_drop.append(i)
            else:
                yield value
        if to_drop:
            for i in reversed(to_drop):
                del iterators[i]
            del to_drop[:]


def yield_from(*iterators):
    for it in iterators:
        yield from it


def bm_plain(N):
    return count_to(COUNT * N)

def bm_round_robin(N):
    i: cython.Py_ssize_t
    return round_robin(*[ count_to(COUNT // i) for i in range(1,N+1) ])

def bm_yield_from(N):
    i: cython.Py_ssize_t
    return yield_from(count_to(N),
                      round_robin(*[ yield_from(count_to(COUNT // i))
                                     for i in range(1,N+1) ]),
                      count_to(N))

def bm_yield_from_nested(N):
    i: cython.Py_ssize_t
    return yield_from(count_to(N),
                      yield_from(count_to(N),
                                 round_robin(*[ yield_from(count_to(COUNT // i))
                                                for i in range(1,N+1) ]),
                                 count_to(N)),
                      count_to(N))


def time_func(fn, N, scale: cython.long = 1, timer=time.perf_counter):
    s: cython.long
    result = None

    begin = timer()
    for s in range(scale):
        result = list(fn(N))
    end = timer()
    return result, end - begin


def benchmark(N, count=10, scale=1, timer=time.perf_counter):
    times = []
    for _ in range(N):
        result, t = time_func(bm_yield_from_nested, count, scale=scale)
        times.append(t)
    return times

main = benchmark


def run_benchmark(repeat=10, scale=1, timer=time.perf_counter):
    return benchmark(repeat, count=200, scale=scale, timer=timer)


if __name__ == "__main__":
    import optparse
    parser = optparse.OptionParser(
        usage="%prog [options]",
        description=("Micro benchmarks for generators."))

    import util
    util.add_standard_options_to(parser)
    options, args = parser.parse_args()

    util.run_benchmark(options, options.num_runs, benchmark)



# mode: compile

cdef struct S:
    int q

def test():
    cdef int i = 1, j = 2, k = 3
    cdef float x = 1, y = 2, z = 3
    cdef object a = 1, b = 2, c = 3, d = 4, e = 5
    cdef int[3] m
    m[0] = 0
    m[1] = 1
    m[2] = 1
    cdef S s = [1]

    global g
    i += j + k
    x += y + z
    x += i
    a += b + c
    g += a
    m[i] += j
    a[i] += b + c
    a[b + c] += d
    (a + b)[c] += d
    a[i : j] += b
    (a + b)[i : j] += c
    a.b += c + d
    (a + b).c += d
    s.q += i



# mode: error
# coding: ASCII
# tag: py_unicode_strings

# ok:
cdef char* c1   =  "abc"
cdef str s1     =  "abc"

cdef unicode u1 = u"abc"
cdef Py_UNICODE* cu1 = u1

cdef bytes b1 = b"abc"
cdef char* c2 = b"abc"

cdef bytes b2 = c1
cdef char* c3 = b1

cdef basestring bs1  =  "abc"
cdef basestring bs2  = u"abc"

cdef object o1  =  "abc"
cdef object o2  = b"abc"
cdef object o3  = u"abc"

o4 = c1
o5 = b1
o6 = s1
o7 = u1
o8 = cu1
o9 = bs1

u1 = bs1
s1 = bs1

# errors:
cdef char* c_f1   = u"\N{SNOWMAN}"  # not bytes compatible
cdef char* c_f2   = u1
cdef char* c_f3   = s1

cdef Py_UNICODE* cu_f1 = c1
cdef Py_UNICODE* cu_f2 = b1
cdef Py_UNICODE* cu_f3 = s1
cdef Py_UNICODE* cu_f4 = b"abc"

cdef bytes b_f1   = u"abc"
cdef bytes b_f2   = u1
cdef bytes b_f3   = s1
cdef bytes b_f4   = bs1

cdef str s_f1  = b"abc"
cdef str s_f2  = b1
cdef str s_f3  = u"abc"
cdef str s_f4  = u1

cdef unicode u_f1 = "abc"
cdef unicode u_f2 = s1
cdef unicode u_f3 = b"abc"
cdef unicode u_f4 = b1
cdef unicode u_f5 = c1

cdef basestring bs_f1 = b"abc"
cdef basestring bs_f2 = b1

cdef tuple t_f1 =  "abc"
cdef tuple t_f2 = u"abc"
cdef tuple t_f3 = b"abc"

cdef list  l_f1 = s1
cdef list  l_f2 = b1
cdef list  l_f3 = u1

print <str>c1
print <str>c1[1:2]
print <unicode>c1
print <unicode>c1[1:2]

cdef void* v_f1   = u"\N{SNOWMAN}"  # not bytes compatible


# ok again:

cdef void* v1 =  "abc"
cdef void* v2 = b"abc"
cdef void* v3 = u"abc"

cdef voidptr_func(void* x): pass

voidptr_func( "abc")
voidptr_func(b"abc")
voidptr_func(u"abc")


_ERRORS = u"""
36:20: Unicode literals do not support coercion to C types other than Py_UCS4/Py_UNICODE (for characters), Py_UNICODE* (for strings) or char*/void* (for auto-encoded strings).
37:20: Unicode objects only support coercion to Py_UNICODE*.
38:20: Unicode objects only support coercion to Py_UNICODE*.

40:25: Cannot assign type 'char *' to 'Py_UNICODE *'
41:25: Cannot convert 'bytes' object to Py_UNICODE*, use 'str'.
43:25: Cannot convert 'bytes' object to Py_UNICODE*, use 'str'.

45:20: Cannot convert Unicode string to 'bytes' implicitly, encoding required.
46:20: Cannot convert Unicode string to 'bytes' implicitly, encoding required.
47:20: Cannot convert Unicode string to 'bytes' implicitly, encoding required.
48:20: Cannot convert Unicode string to 'bytes' implicitly, encoding required.

50:17: Cannot convert 'bytes' object to str implicitly, decoding required
51:17: Cannot convert 'bytes' object to str implicitly, decoding required

57:20: Cannot convert 'bytes' object to str implicitly, decoding required
58:20: Cannot convert 'bytes' object to str implicitly, decoding required
59:20: Cannot convert 'char*' to unicode implicitly, decoding required

61:24: Cannot convert 'bytes' object to str implicitly, decoding required
62:24: Cannot convert 'bytes' object to str implicitly, decoding required

64:19: Cannot assign type 'str object' to 'tuple object'
65:18: Cannot assign type 'str object' to 'tuple object'
66:18: Cannot assign type 'bytes object' to 'tuple object'

72:11: Cannot convert 'char*' to unicode implicitly, decoding required
73:13: default encoding required for conversion from 'char *' to 'str object'
74:15: Cannot convert 'char*' to unicode implicitly, decoding required
75:17: default encoding required for conversion from 'char *' to 'str object'
77:20: Unicode literals do not support coercion to C types other than Py_UCS4/Py_UNICODE (for characters), Py_UNICODE* (for strings) or char*/void* (for auto-encoded strings).
"""



# mode: compile

def f():
    from spam import eggs
    from spam.morespam import bacon, eggs, ham
    from spam import eggs as ova
    from . import spam
    from ... import spam
    from .. . import spam
    from . .. import spam
    from . . . import spam
    from .. import spam, foo
    from . . import spam, foo
    from ... import spam, foobar
    from .. . import spam, foobar
    from . .. import spam, foobar
    from . . . import spam, foobar
    from .spam import foo
    from . spam import foo
    from ...spam import foo, bar
    from .. . spam import foo, bar
    from . .. spam import foo, bar
    from . . . spam import foo, bar
    from ...spam.foo import bar
    from ... spam.foo import bar
    from .. . spam.foo import bar
    from . .. spam.foo import bar
    from . . . spam.foo import bar
    from ...spam.foo import foo, bar
    from ... spam.foo import foo, bar
    from .. . spam.foo import foo, bar
    from . .. spam.foo import foo, bar
    from . . . spam.foo import foo, bar
    from ...spam.foo import (foo, bar)
    from ... spam.foo import (foo, bar)
    from .. . spam.foo import (foo, bar)
    from .. . spam.foo import (foo, bar)
    from . .. spam.foo import (foo, bar)
    from . . . spam.foo import (foo, bar)



# ticket: t431

__doc__ = u"""
>>> s == s_interned
True
>>> s == s_interned_dynamic
True
>>> s == 'abc' == s_interned == s_interned_dynamic
True
"""


s = 'abc'

s_interned = intern(s)

s_interned_dynamic = intern('a'+'b'+'c')



# mode: compile

# This caused a "maximum recursion depth exceeded" at some point,
# see https://github.com/cython/cython/issues/1744

cdef inline bint g(int x, int y): return True

cdef cython_bug(int u):
    try:
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
        g(u, u)
    finally:
        g(u, u)

cython_bug(1)



# tests copied from test/test_bool.py in Py3.14

from contextlib import contextmanager


cdef assertEqual(a,b):
    assert a == b, '%r != %r' % (a,b)

cdef assertGreaterEqual(a,b):
    assert a >= b, '%r < %r' % (a,b)

cdef assertIs(a,b):
    assert a is b, '%r is not %r' % (a,b)

cdef assertIsNot(a,b):
    assert a is not b, '%r is %r' % (a,b)

cdef assertNotIsInstance(a,b):
    assert not isinstance(a,b), 'isinstance(%r, %s)' % (a,b)

def assertRaises(exc_type, func=None, *args):
    if func is not None:
        try:
            func(*args)
        except exc_type as exc:
            pass
        else:
            assert False, f"Exception {exc_type} was not raised"
    else:
        @contextmanager
        def cm(exc_type):
            try:
                yield
            except exc_type as exc:
                pass
            else:
                assert False, f"Exception {exc_type} was not raised"

        return cm(exc_type)

@contextmanager
def assertRaisesRegex(exc_type, regex):
    try:
        yield
    except exc_type as exc:
        # Ignore regex
        pass
    else:
        assert False, f"Exception {exc_type} was not raised"


def test_repr():
    """
    >>> test_repr()
    """
    assertEqual(repr(False), 'False')
    assertEqual(repr(True), 'True')
    assertIs(eval(repr(False)), False)
    assertIs(eval(repr(True)), True)


def test_str():
    """
    >>> test_str()
    """
    assertEqual(str(False), 'False')
    assertEqual(str(True), 'True')


def test_int():
    """
    >>> test_int()
    """
    assertEqual(int(False), 0)
    assertIsNot(int(False), False)
    assertEqual(int(True), 1)
    assertIsNot(int(True), True)


def test_float():
    """
    >>> test_float()
    """
    assertEqual(float(False), 0.0)
    assertIsNot(float(False), False)
    assertEqual(float(True), 1.0)
    assertIsNot(float(True), True)


def test_complex():
    """
    >>> test_complex()
    """
    assertEqual(complex(False), 0j)
    assertEqual(complex(False), False)
    assertEqual(complex(True), 1+0j)
    assertEqual(complex(True), True)


def test_math():
    """
    >>> test_math()
    """
    assertEqual(+False, 0)
    assertIsNot(+False, False)
    assertEqual(-False, 0)
    assertIsNot(-False, False)
    assertEqual(abs(False), 0)
    assertIsNot(abs(False), False)
    assertEqual(+True, 1)
    assertIsNot(+True, True)
    assertEqual(-True, -1)
    assertEqual(abs(True), 1)
    assertIsNot(abs(True), True)

    '''
    with assertWarns(DeprecationWarning):
        # We need to put the bool in a variable, because the constant
        # ~False is evaluated at compile time due to constant folding;
        # consequently the DeprecationWarning would be issued during
        # module loading and not during test execution.
        false = False
        assertEqual(~false, -1)
    with assertWarns(DeprecationWarning):
        # also check that the warning is issued in case of constant
        # folding at compile time
        assertEqual(eval("~False"), -1)
    with assertWarns(DeprecationWarning):
        true = True
        assertEqual(~true, -2)
    with assertWarns(DeprecationWarning):
        assertEqual(eval("~True"), -2)
    '''

    assertEqual(False+2, 2)
    assertEqual(True+2, 3)
    assertEqual(2+False, 2)
    assertEqual(2+True, 3)

    assertEqual(False+False, 0)
    assertIsNot(False+False, False)
    assertEqual(False+True, 1)
    assertIsNot(False+True, True)
    assertEqual(True+False, 1)
    assertIsNot(True+False, True)
    assertEqual(True+True, 2)

    assertEqual(True-True, 0)
    assertIsNot(True-True, False)
    assertEqual(False-False, 0)
    assertIsNot(False-False, False)
    assertEqual(True-False, 1)
    assertIsNot(True-False, True)
    assertEqual(False-True, -1)

    assertEqual(True*1, 1)
    assertEqual(False*1, 0)
    assertIsNot(False*1, False)

    assertEqual(True/1, 1)
    assertIsNot(True/1, True)
    assertEqual(False/1, 0)
    assertIsNot(False/1, False)

    assertEqual(True%1, 0)
    assertIsNot(True%1, False)
    assertEqual(True%2, 1)
    assertIsNot(True%2, True)
    assertEqual(False%1, 0)
    assertIsNot(False%1, False)

    for b in False, True:
        for i in 0, 1, 2:
            assertEqual(b**i, int(b)**i)
            assertIsNot(b**i, bool(int(b)**i))

    for a in False, True:
        for b in False, True:
            assertIs(a&b, bool(int(a)&int(b)))
            assertIs(a|b, bool(int(a)|int(b)))
            assertIs(a^b, bool(int(a)^int(b)))
            assertEqual(a&int(b), int(a)&int(b))
            assertIsNot(a&int(b), bool(int(a)&int(b)))
            assertEqual(a|int(b), int(a)|int(b))
            assertIsNot(a|int(b), bool(int(a)|int(b)))
            assertEqual(a^int(b), int(a)^int(b))
            assertIsNot(a^int(b), bool(int(a)^int(b)))
            assertEqual(int(a)&b, int(a)&int(b))
            assertIsNot(int(a)&b, bool(int(a)&int(b)))
            assertEqual(int(a)|b, int(a)|int(b))
            assertIsNot(int(a)|b, bool(int(a)|int(b)))
            assertEqual(int(a)^b, int(a)^int(b))
            assertIsNot(int(a)^b, bool(int(a)^int(b)))

    assertIs(1==1, True)
    assertIs(1==0, False)
    assertIs(0<1, True)
    assertIs(1<0, False)
    assertIs(0<=0, True)
    assertIs(1<=0, False)
    assertIs(1>0, True)
    assertIs(1>1, False)
    assertIs(1>=1, True)
    assertIs(0>=1, False)
    assertIs(0!=1, True)
    assertIs(0!=0, False)

    x = [1]
    assertIs(x is x, True)
    assertIs(x is not x, False)

    assertIs(1 in x, True)
    assertIs(0 in x, False)
    assertIs(1 not in x, False)
    assertIs(0 not in x, True)

    x = {1: 2}
    assertIs(x is x, True)
    assertIs(x is not x, False)

    assertIs(1 in x, True)
    assertIs(0 in x, False)
    assertIs(1 not in x, False)
    assertIs(0 not in x, True)

    assertIs(not True, False)
    assertIs(not False, True)


def test_convert():
    """
    >>> test_convert()
    """
    assertRaises(TypeError, bool, 42, 42)
    assertIs(bool(10), True)
    assertIs(bool(1), True)
    assertIs(bool(-1), True)
    assertIs(bool(0), False)
    assertIs(bool("hello"), True)
    assertIs(bool(""), False)
    assertIs(bool(), False)


def test_keyword_args():
    """
    >>> test_keyword_args()
    """
    with assertRaisesRegex(TypeError, 'keyword argument'):
        bool(x=10)


def test_format():
    """
    >>> test_format()
    """
    assertEqual("%d" % False, "0")
    assertEqual("%d" % True, "1")
    assertEqual("%x" % False, "0")
    assertEqual("%x" % True, "1")


def test_hasattr():
    """
    >>> test_hasattr()
    """
    assertIs(hasattr([], "append"), True)
    assertIs(hasattr([], "wobble"), False)


def test_callable():
    """
    >>> test_callable()
    """
    assertIs(callable(len), True)
    assertIs(callable(1), False)


def test_isinstance():
    """
    >>> test_isinstance()
    """
    assertIs(isinstance(True, bool), True)
    assertIs(isinstance(False, bool), True)
    assertIs(isinstance(True, int), True)
    assertIs(isinstance(False, int), True)
    assertIs(isinstance(1, bool), False)
    assertIs(isinstance(0, bool), False)


def test_issubclass():
    """
    >>> test_issubclass()
    """
    assertIs(issubclass(bool, int), True)
    assertIs(issubclass(int, bool), False)


def test_contains():
    """
    >>> test_contains()
    """
    assertIs(1 in {}, False)
    assertIs(1 in {1:1}, True)


def test_string():
    """
    >>> test_string()
    """
    assertIs("xyz".endswith("z"), True)
    assertIs("xyz".endswith("x"), False)
    assertIs("xyz0123".isalnum(), True)
    assertIs("@#$%".isalnum(), False)
    assertIs("xyz".isalpha(), True)
    assertIs("@#$%".isalpha(), False)
    assertIs("0123".isdigit(), True)
    assertIs("xyz".isdigit(), False)
    assertIs("xyz".islower(), True)
    assertIs("XYZ".islower(), False)
    assertIs("0123".isdecimal(), True)
    assertIs("xyz".isdecimal(), False)
    assertIs("0123".isnumeric(), True)
    assertIs("xyz".isnumeric(), False)
    assertIs(" ".isspace(), True)
    assertIs("\xa0".isspace(), True)
    assertIs("\u3000".isspace(), True)
    assertIs("XYZ".isspace(), False)
    assertIs("X".istitle(), True)
    assertIs("x".istitle(), False)
    assertIs("XYZ".isupper(), True)
    assertIs("xyz".isupper(), False)
    assertIs("xyz".startswith("x"), True)
    assertIs("xyz".startswith("z"), False)


def test_boolean():
    """
    >>> test_boolean()
    """
    assertEqual(True & 1, 1)
    assertNotIsInstance(True & 1, bool)
    assertIs(True & True, True)

    assertEqual(True | 1, 1)
    assertNotIsInstance(True | 1, bool)
    assertIs(True | True, True)

    assertEqual(True ^ 1, 0)
    assertNotIsInstance(True ^ 1, bool)
    assertIs(True ^ True, False)


'''
# Useless in Cython and requires 'os_helper'.
def test_fileclosed():
    """
    >>> test_fileclosed()
    """
    try:
        with open(os_helper.TESTFN, "w", encoding="utf-8") as f:
            assertIs(f.closed, False)
        assertIs(f.closed, True)
    finally:
        os.remove(os_helper.TESTFN)
'''


def test_types():
    """
    >>> test_types()
    """
    # types are always true.
    for t in [bool, complex, dict, float, int, list, object,
                set, str, tuple, type]:
        assertIs(bool(t), True)


def test_operator():
    """
    >>> test_operator()
    """
    import operator
    assertIs(operator.truth(0), False)
    assertIs(operator.truth(1), True)
    assertIs(operator.not_(1), False)
    assertIs(operator.not_(0), True)
    assertIs(operator.contains([], 1), False)
    assertIs(operator.contains([1], 1), True)
    assertIs(operator.lt(0, 0), False)
    assertIs(operator.lt(0, 1), True)
    assertIs(operator.is_(True, True), True)
    assertIs(operator.is_(True, False), False)
    assertIs(operator.is_not(True, True), False)
    assertIs(operator.is_not(True, False), True)


def test_marshal():
    """
    >>> test_marshal()
    """
    import marshal
    assertIs(marshal.loads(marshal.dumps(True)), True)
    assertIs(marshal.loads(marshal.dumps(False)), False)


def test_pickle():
    """
    >>> test_pickle()
    """
    import pickle
    for proto in range(pickle.HIGHEST_PROTOCOL + 1):
        assertIs(pickle.loads(pickle.dumps(True, proto)), True)
        assertIs(pickle.loads(pickle.dumps(False, proto)), False)


def test_picklevalues():
    """
    >>> test_picklevalues()
    """
    # Test for specific backwards-compatible pickle values
    import pickle
    assertEqual(pickle.dumps(True, protocol=0), b"I01\n.")
    assertEqual(pickle.dumps(False, protocol=0), b"I00\n.")
    assertEqual(pickle.dumps(True, protocol=1), b"I01\n.")
    assertEqual(pickle.dumps(False, protocol=1), b"I00\n.")
    assertEqual(pickle.dumps(True, protocol=2), b'\x80\x02\x88.')
    assertEqual(pickle.dumps(False, protocol=2), b'\x80\x02\x89.')


def test_convert_to_bool():
    """
    >>> test_convert_to_bool()
    """
    # Verify that TypeError occurs when bad things are returned
    # from __bool__().  This isn't really a bool test, but
    # it's related.
    check = lambda o: assertRaises(TypeError, bool, o)
    class Foo(object):
        def __bool__(self):
            return self
    check(Foo())

    class Bar(object):
        def __bool__(self):
            return "Yes"
    check(Bar())

    class Baz(int):
        def __bool__(self):
            return self
    check(Baz())

    # __bool__() must return a bool not an int
    class Spam(int):
        def __bool__(self):
            return 1
    check(Spam())

    class Eggs:
        def __len__(self):
            return -1
    assertRaises(ValueError, bool, Eggs())


def test_interpreter_convert_to_bool_raises():
    """
    >>> test_interpreter_convert_to_bool_raises()
    """
    class SymbolicBool:
        def __bool__(self):
            raise TypeError

    class Symbol:
        def __gt__(self, other):
            return SymbolicBool()

    x = Symbol()

    with assertRaises(TypeError):
        if x > 0:
            msg = "x > 0 was true"
        else:
            msg = "x > 0 was false"

    # This used to create negative refcounts, see gh-102250
    del x


def test_from_bytes():
    """
    >>> test_from_bytes()
    """
    assertIs(bool.from_bytes(b'\x00'*8, 'big'), False)
    assertIs(bool.from_bytes(b'abcd', 'little'), True)


def test_sane_len():
    """
    >>> test_sane_len()
    """
    # this test just tests our assumptions about __len__
    # this will start failing if __len__ changes assertions
    for badval in ['illegal', -1, 1 << 32]:
        class A:
            def __len__(self):
                return badval
        try:
            bool(A())
        except (Exception) as e_bool:
            try:
                len(A())
            except (Exception) as e_len:
                assertEqual(str(e_bool), str(e_len))


def test_blocked():
    """
    >>> test_blocked()
    """
    class A:
        __bool__ = None
    assertRaises(TypeError, bool, A())

    class B:
        def __len__(self):
            return 10
        __bool__ = None
    assertRaises(TypeError, bool, B())

    class C:
        __len__ = None
    assertRaises(TypeError, bool, C())


def test_real_and_imag():
    """
    >>> test_real_and_imag()
    """
    assertEqual(True.real, 1)
    assertEqual(True.imag, 0)
    assertIs(type(True.real), int)
    assertIs(type(True.imag), int)
    assertEqual(False.real, 0)
    assertEqual(False.imag, 0)
    assertIs(type(False.real), int)
    assertIs(type(False.imag), int)


def test_bool_called_at_least_once():
    """
    >>> test_bool_called_at_least_once()
    """
    class X:
        def __init__(self):
            self.count = 0
        def __bool__(self):
            self.count += 1
            return True

    def f(x):
        if x or True:
            pass

    x = X()
    f(x)
    assertGreaterEqual(x.count, 1)


def test_bool_new():
    """
    >>> test_bool_new()
    """
    assertIs(bool.__new__(bool), False)
    assertIs(bool.__new__(bool, 1), True)
    assertIs(bool.__new__(bool, 0), False)
    assertIs(bool.__new__(bool, False), False)
    assertIs(bool.__new__(bool, True), True)



# mode: run
# cython: language_level=3

import cython
from cython import sizeof

is_compiled = cython.compiled

NULL = 5
_NULL = NULL


def test_sizeof():
    """
    >>> test_sizeof()
    True
    True
    True
    True
    True
    """
    x = cython.declare(cython.bint)
    print(cython.sizeof(x) == cython.sizeof(cython.bint))
    print(sizeof(cython.char) <= sizeof(cython.short) <= sizeof(cython.int) <= sizeof(cython.long) <= sizeof(cython.longlong))
    print(cython.sizeof(cython.uint) == cython.sizeof(cython.int))
    print(cython.sizeof(cython.p_int) == cython.sizeof(cython.p_double))
    if cython.compiled:
        print(cython.sizeof(cython.char) < cython.sizeof(cython.longlong))
    else:
        print(cython.sizeof(cython.char) == 1)


def test_declare(n):
    """
    >>> test_declare(100)
    (100, 100, 100)
    >>> test_declare(100.5)
    (100, 100, 100)
    """
    x = cython.declare(cython.int)
    y = cython.declare(cython.int, n)
    z = cython.declare(int, n)  # C int
    if cython.compiled:
        cython.declare(xx=cython.int, yy=cython.long)
        i = cython.sizeof(xx)
    ptr = cython.declare(cython.p_int, cython.address(y))
    return y, z, ptr[0]


@cython.locals(x=cython.double, n=cython.int)
def test_cast(x):
    """
    >>> test_cast(1.5)
    1
    """
    n = cython.cast(cython.int, x)
    return n


@cython.locals(x=cython.int, y=cython.p_int)
def test_address(x):
    """
    >>> test_address(39)
    39
    """
    y = cython.address(x)
    return y[0]


@cython.wraparound(False)
def test_wraparound(x):
    """
    >>> test_wraparound([1, 2, 3])
    [1, 2, 1]
    """
    with cython.wraparound(True):
        x[-1] = x[0]
    return x


@cython.boundscheck(False)
def test_boundscheck(x):
    """
    >>> test_boundscheck([1, 2, 3])
    3
    >>> try: test_boundscheck([1, 2])
    ... except IndexError: pass
    """
    with cython.boundscheck(True):
        return x[2]


## CURRENTLY BROKEN - FIXME!!
## Is this test make sense? Implicit conversion in pure Python??

## @cython.locals(x=cython.int)
## @cython.locals(y=cython.bint)
## def test_locals(x):
##     """
##     >>> test_locals(5)
##     True
##     """
##     y = x
##     return y


def test_with_nogil(nogil, should_raise=False):
    """
    >>> raised = []
    >>> class nogil(object):
    ...     def __enter__(self):
    ...         pass
    ...     def __exit__(self, exc_class, exc, tb):
    ...         raised.append(exc)
    ...         return exc_class is None

    >>> test_with_nogil(nogil())
    WORKS
    True
    >>> raised
    [None]

    >>> test_with_nogil(nogil(), should_raise=True)
    Traceback (most recent call last):
    ValueError: RAISED!

    >>> raised[1] is None
    False
    """
    result = False
    should_raise_bool = True if should_raise else False  # help the type inference ...
    with nogil:
        print("WORKS")
        with cython.nogil:
            result = True
            if should_raise_bool:
                raise ValueError("RAISED!")
    return result


MyUnion = cython.union(n=cython.int, x=cython.double)
MyStruct = cython.struct(is_integral=cython.bint, data=MyUnion)
MyStruct2 = cython.typedef(MyStruct[2])
MyStruct3 = cython.typedef(MyStruct[3])

def test_struct(n, x):
    """
    >>> test_struct(389, 1.64493)
    (389, 1.64493, False)
    """
    a = cython.declare(MyStruct3)
    a[0] = MyStruct(is_integral=True, data=MyUnion(n=n))
    a[1] = MyStruct(is_integral=False, data={'x': x})
    # dict is ordered => struct creation via keyword arguments above was deterministic!
    a[2] = MyStruct(False, MyUnion(x=x))
    return a[0].data.n, a[1].data.x, a[2].is_integral

import cython as cy
from cython import declare, cast, locals, address, typedef, p_void, compiled
from cython import declare as my_declare, locals as my_locals, p_void as my_void_star, typedef as my_typedef, compiled as my_compiled

@my_locals(a=cython.p_void)
def test_imports():
    """
    >>> test_imports()
    (True, True)
    """
    a = cython.NULL
    b = declare(p_void, cython.NULL)
    c = my_declare(my_void_star, cython.NULL)
    d = cy.declare(cy.p_void, cython.NULL)

    return a == d, compiled == my_compiled

## CURRENTLY BROKEN - FIXME!!

# MyStruct3 = typedef(MyStruct[3])
# MyStruct4 = my_typedef(MyStruct[4])
# MyStruct5 = cy.typedef(MyStruct[5])

def test_declare_c_types(n):
    """
    >>> test_declare_c_types(0)
    >>> test_declare_c_types(1)
    >>> test_declare_c_types(2)
    """
    #
    b00 = cython.declare(cython.bint, 0)
    b01 = cython.declare(cython.bint, 1)
    b02 = cython.declare(cython.bint, 2)
    #
    i00 = cython.declare(cython.uchar, n)
    i01 = cython.declare(cython.char, n)
    i02 = cython.declare(cython.schar, n)
    i03 = cython.declare(cython.ushort, n)
    i04 = cython.declare(cython.short, n)
    i05 = cython.declare(cython.sshort, n)
    i06 = cython.declare(cython.uint, n)
    i07 = cython.declare(cython.int, n)
    i08 = cython.declare(cython.sint, n)
    i09 = cython.declare(cython.slong, n)
    i10 = cython.declare(cython.long, n)
    i11 = cython.declare(cython.ulong, n)
    i12 = cython.declare(cython.slonglong, n)
    i13 = cython.declare(cython.longlong, n)
    i14 = cython.declare(cython.ulonglong, n)

    i20 = cython.declare(cython.Py_ssize_t, n)
    i21 = cython.declare(cython.size_t, n)
    #
    f00 = cython.declare(cython.float, n)
    f01 = cython.declare(cython.double, n)
    f02 = cython.declare(cython.longdouble, n)
    #
    #z00 = cython.declare(cython.complex, n+1j)
    #z01 = cython.declare(cython.floatcomplex, n+1j)
    #z02 = cython.declare(cython.doublecomplex, n+1j)
    #z03 = cython.declare(cython.longdoublecomplex, n+1j)


@cython.ccall
@cython.returns(cython.double)
def c_call(x):
    if x == -2.0:
        raise RuntimeError("huhu!")
    return x


def call_ccall(x):
    """
    Test that a declared return type is honoured when compiled.

    >>> result, return_type = call_ccall(1)

    >>> (not is_compiled and 'double') or return_type
    'double'
    >>> (is_compiled and 'int') or return_type
    'int'

    >>> (not is_compiled and 1.0) or result
    1.0
    >>> (is_compiled and 1) or result
    1

    >>> call_ccall(-2)
    Traceback (most recent call last):
    RuntimeError: huhu!
    """
    ret = c_call(x)
    return ret, cython.typeof(ret)


@cython.cfunc
@cython.inline
@cython.returns(cython.double)
def cdef_inline(x):
    if x == -2.0:
        raise RuntimeError("huhu!")
    return x + 1


def call_cdef_inline(x):
    """
    >>> result, return_type = call_cdef_inline(1)
    >>> (not is_compiled and 'float') or type(result).__name__
    'float'
    >>> (not is_compiled and 'double') or return_type
    'double'
    >>> (is_compiled and 'int') or return_type
    'int'
    >>> result == 2.0  or  result
    True

    >>> call_cdef_inline(-2)
    Traceback (most recent call last):
    RuntimeError: huhu!
    """
    ret = cdef_inline(x)
    return ret, cython.typeof(ret)


@cython.cfunc
@cython.nogil
@cython.locals(x=cython.int)
@cython.returns(cython.int)
def cdef_nogil(x):
    return x + 1


@cython.cfunc
@cython.nogil(True)
@cython.locals(x=cython.int)
@cython.returns(cython.int)
def cdef_nogil_true(x):
    return x + 1


@cython.cfunc
@cython.nogil(False)
@cython.locals(x=cython.int)
@cython.returns(cython.int)
def cdef_nogil_false(x):
    return x + 1


@cython.locals(x=cython.int, result=cython.int)
def test_cdef_nogil(x):
    """
    >>> test_cdef_nogil(5)
    24
    """
    with cython.nogil:
        result = cdef_nogil(x)
    with cython.nogil(True):
        result += cdef_nogil_true(x)
    with cython.nogil(False), cython.nogil(True), cython.gil(False):
        result += cdef_nogil(x)
    result += cdef_nogil_false(x)
    return result


@cython.cfunc
@cython.inline
def has_inner_func(x):
    # the inner function must remain a Python function
    # (and inline must not be applied to it)
    @cython.test_fail_if_path_exists("//CFuncDefNode")
    def inner():
        return x
    return inner


def test_has_inner_func():
    """
    >>> test_has_inner_func()
    1
    """
    return has_inner_func(1)()


@cython.locals(counts=cython.int[10], digit=cython.int)
def count_digits_in_carray(digits):
    """
    >>> digits = '37692837651902834128342341'
    >>> ''.join(sorted(digits))
    '01112222333334445667788899'
    >>> count_digits_in_carray(map(int, digits))
    [1, 3, 4, 5, 3, 1, 2, 2, 3, 2]
    """
    counts = [0] * 10
    for digit in digits:
        assert 0 <= digit <= 9
        counts[digit] += 1
    return counts


@cython.test_assert_path_exists("//CFuncDeclaratorNode//IntNode[@base_10_value = '-1']")
@cython.ccall
@cython.returns(cython.long)
@cython.exceptval(-1)
def ccall_except(x):
    """
    >>> ccall_except(41)
    42
    >>> ccall_except(0)
    Traceback (most recent call last):
    ValueError
    """
    if x == 0:
        raise ValueError
    return x+1


@cython.test_assert_path_exists("//CFuncDeclaratorNode//IntNode[@base_10_value = '-1']")
@cython.cfunc
@cython.returns(cython.long)
@cython.exceptval(-1)
def cdef_except(x):
    if x == 0:
        raise ValueError
    return x+1


def call_cdef_except(x):
    """
    >>> call_cdef_except(41)
    42
    >>> call_cdef_except(0)
    Traceback (most recent call last):
    ValueError
    """
    return cdef_except(x)


@cython.test_assert_path_exists("//CFuncDeclaratorNode//IntNode[@base_10_value = '-1']")
@cython.ccall
@cython.returns(cython.long)
@cython.exceptval(-1, check=True)
def ccall_except_check(x):
    """
    >>> ccall_except_check(41)
    42
    >>> ccall_except_check(-2)
    -1
    >>> ccall_except_check(0)
    Traceback (most recent call last):
    ValueError
    """
    if x == 0:
        raise ValueError
    return x+1


@cython.test_assert_path_exists("//CFuncDeclaratorNode")
@cython.ccall
@cython.returns(cython.long)
@cython.exceptval(check=True)
def ccall_except_check_always(x):
    """
    Note that this actually takes the same shortcut as for a Cython-syntax cdef function
    and does except ?-1

    >>> ccall_except_check_always(41)
    42
    >>> ccall_except_check_always(0)
    Traceback (most recent call last):
    ValueError
    """
    if x == 0:
        raise ValueError
    return x+1


@cython.test_fail_if_path_exists("//CFuncDeclaratorNode//IntNode[@base_10_value = '-1']")
@cython.test_assert_path_exists("//CFuncDeclaratorNode")
@cython.ccall
@cython.returns(cython.long)
@cython.exceptval(check=False)
def ccall_except_no_check(x):
    """
    >>> ccall_except_no_check(41)
    42
    >>> try: _ = ccall_except_no_check(0)  # no exception propagated!
    ... except ValueError: assert not is_compiled
    """
    if x == 0:
        raise ValueError
    return x+1


@cython.final
@cython.cclass
class CClass(object):
    """
    >>> c = CClass(2)
    >>> c.get_attr()
    int
    2
    """
    cython.declare(attr=cython.int)

    def __init__(self, attr):
        self.attr = attr

    def get_attr(self):
        print(cython.typeof(self.attr))
        return self.attr


class TestUnboundMethod:
    """
    >>> C = TestUnboundMethod
    >>> C.meth is C.__dict__["meth"]
    True
    """
    def meth(self): pass

@cython.cclass
class Foo:
    a = cython.declare(cython.double)
    b = cython.declare(cython.double)
    c = cython.declare(cython.double)

    @cython.locals(a=cython.double, b=cython.double, c=cython.double)
    def __init__(self, a, b, c):
        self.a = a
        self.b = b
        self.c = c

@cython.cclass
class EmptyClass(object):
    def __init__(self, *args):
        pass

def same_type_cast():
    """
    >>> same_type_cast()
    True
    """

    f = EmptyClass()
    return f is cython.cast(EmptyClass, f)

def multi_args_init_cast():
    """
    >>> multi_args_init_cast()
    True
    """
    f = Foo(10, 20, 30)
    return cython.cast(Foo, f) is f

def multi_args_init_declare():
    """
    >>> multi_args_init_declare() is None
    True
    """
    f = cython.declare(Foo)

    if cython.compiled:
        f = None

    return f

EmptyClassSyn = cython.typedef(EmptyClass)

def empty_declare():
    """
    >>> empty_declare()
    []
    """

    r0 = cython.declare(EmptyClass)
    r1 = cython.declare(EmptyClassSyn)
    r2 = cython.declare(MyStruct)
    r3 = cython.declare(MyUnion)
    r4 = cython.declare(MyStruct2)
    r5 = cython.declare(cython.int[2])

    if cython.compiled:
        r0 = None
        r1 = None

    res = [
        r0 is None,
        r1 is None,
        r2 is not None,
        r3 is not None,
        r4 is not None,
        r5 is not None
    ]

    r2.is_integral = True
    assert r2.is_integral == True

    r3.x = 12.3
    assert r3.x == 12.3

    #It generates a correct C code, but raises an exception when interpreted
    if cython.compiled:
        r4[0].is_integral = True
        assert r4[0].is_integral == True

    r5[0] = 42
    assert r5[0] == 42

    return [i for i, x in enumerate(res) if not x]

def same_declare():
    """
    >>> same_declare()
    True
    """

    f = EmptyClass()
    f2 = cython.declare(EmptyClass, f)
    return f2 is f

def none_cast():
    """
    >>> none_cast() is None
    True
    """

    f = None
    return cython.cast(EmptyClass, f)

def none_declare():
    """
    >>> none_declare() is None
    True
    """

    f = None
    f2 = cython.declare(Foo, f)
    return f2

def array_init_with_list():
    """
    >>> array_init_with_list()
    [10, 42]
    """
    x = cython.declare(cython.int[20], list(range(20)))
    x[12] = 42

    return [x[10], x[12]]



# mode: run

cimport cython


def _bool():
    """
    >>> _bool() == bool()
    True
    """
    return bool()

def _int():
    """
    >>> _int() == int()
    True
    """
    return int()

def _float():
    """
    >>> _float() == float()
    True
    """
    return float()

def _complex():
    """
    >>> _complex() == complex()
    True
    """
    return complex()

def _bytes():
    """
    >>> _bytes() == bytes()
    True
    """
    return bytes()

def _str():
    """
    >>> _str() == str()
    True
    """
    return str()

def _unicode():
    """
    >>> _unicode() == str()
    True
    """
    return unicode()

def _tuple():
    """
    >>> _tuple() == tuple()
    True
    """
    return tuple()

def _list():
    """
    >>> _list() == list()
    True
    """
    return list()

def _dict():
    """
    >>> _dict() == dict()
    True
    """
    return dict()

py_set = cython.set

def _set():
    """
    >>> _set() == py_set()
    True
    """
    return set()

py_frozenset = cython.frozenset

def _frozenset():
    """
    >>> _frozenset() == py_frozenset()
    True
    """
    return frozenset()



from cython cimport typeof

def call2():
    """
    >>> call2()
    """
    b(1,2)

def call3():
    """
    >>> call3()
    """
    b(1,2,3)

def call4():
    """
    >>> call4()
    """
    b(1,2,3,4)

# the called function:

cdef b(a, b, c=1, d=2):
    pass


cdef int foo(int a, int b=1, int c=1):
    return a+b*c

def test_foo():
    """
    >>> test_foo()
    2
    3
    7
    26
    """
    print foo(1)
    print foo(1, 2)
    print foo(1, 2, 3)
    print foo(1, foo(2, 3), foo(4))

cdef class A:
    cpdef method(self):
        """
        >>> A().method()
        'A'
        """
        return typeof(self)

cdef class B(A):
    cpdef method(self, int x = 0):
        """
        >>> B().method()
        ('B', 0)
        >>> B().method(100)
        ('B', 100)
        """
        return typeof(self), x

cdef class C(B):
    cpdef method(self, int x = 10):
        """
        >>> C().method()
        ('C', 10)
        >>> C().method(100)
        ('C', 100)
        """
        return typeof(self), x



from __future__ import unicode_literals
import struct

# Tests buffer format string parsing.


from libc cimport stdlib

def little_endian():
    cdef unsigned int n = 1
    return (<unsigned char*>&n)[0] != 0

if little_endian():
    current_endian = '<'
    other_endian = '>'
else:
    current_endian = '>'
    other_endian = '<'

cdef struct align_of_float_helper:
    char ch
    float d
cdef struct align_of_int_helper:
    char ch
    int i
float_align = sizeof(align_of_float_helper) - sizeof(float)
int_align = sizeof(align_of_int_helper) - sizeof(int)
if float_align != 4 or sizeof(float) != 4:
    raise RuntimeError("Alignment or size of float is %d on this system, please report to cython-dev for a testcase fix" % float_align)
if int_align != 4 or sizeof(int) != 4:
    raise RuntimeError("Alignment or size of int is %d on this system, please report to cython-dev for a testcase fix" % int_align)


cdef class MockBuffer:
    cdef Py_ssize_t zero
    cdef Py_ssize_t minusone
    cdef bytes format
    cdef object itemsize

    def __init__(self, format, itemsize):
        self.format = unicode(format).encode(u"ASCII")
        self.itemsize = itemsize
        self.zero = 0
        self.minusone = -1

    def __getbuffer__(self, Py_buffer* info, int flags):
        info.buf = NULL
        info.strides = &self.zero
        info.suboffsets = &self.minusone
        info.shape = &self.zero
        info.ndim = 1
        info.format = self.format
        info.itemsize = self.itemsize

def _int(fmt):
    """
    >>> _int("i")

    >>> _int("b")
    Traceback (most recent call last):
       ...
    ValueError: Buffer dtype mismatch, expected 'int' but got 'signed char'

    >>> _int("if")
    Traceback (most recent call last):
       ...
    ValueError: Buffer dtype mismatch, expected end but got 'float'

    >>> _int("$$")
    Traceback (most recent call last):
       ...
    ValueError: Does not understand character buffer dtype format string ('$')
    """
    cdef object[int] buf = MockBuffer(fmt, sizeof(int))

def _ulong(fmt):
    """
    >>> _ulong("L")
    """
    cdef object[unsigned long] buf = MockBuffer(fmt, sizeof(unsigned long))

def wrongsize():
    """
    >>> wrongsize()
    Traceback (most recent call last):
       ...
    ValueError: Item size of buffer (1 byte) does not match size of 'float' (4 bytes)

    """
    cdef object[float] buf = MockBuffer("f", 1)

def _obj(fmt):
    """
    >>> _obj("O")
    >>> _obj("i")
    Traceback (most recent call last):
       ...
    ValueError: Buffer dtype mismatch, expected 'Python object' but got 'int'
    """
    cdef object[object] buf = MockBuffer(fmt, sizeof(void*))


cdef struct ComplexFloat:
    float real
    float imag

ctypedef struct Char3Int:
    char a
    int b
    int c
    int d

ctypedef struct LongString:
    char[90198] c

cdef struct CharIntCFloat:
    char a
    int b
    ComplexFloat c
    float d

cdef struct UnpackedStruct1:
    char a
    int b
    ComplexFloat c
    float c2
    Char3Int d

ctypedef struct UnpackedStruct2:
    CharIntCFloat a
    Char3Int b

ctypedef struct UnpackedStruct3:
    CharIntCFloat a
    char b
    int c, d, e

cdef struct UnpackedStruct4:
    char a
    int b
    ComplexFloat c
    float c2
    char d
    int e, f, g

def char3int(fmt):
    """
    >>> char3int("ciii")
    >>> char3int("c1i1i1i")
    >>> char3int("c3i")
    >>> char3int("ci2i")

    >>> char3int("c@i@2i")

    Extra pad bytes (assuming int size is 4 or more)
    >>> char3int("cxiii")
    >>> char3int("c3xiii")
    >>> char3int("cxxxiii")

    Standard alignment (assuming int size is 4)
    >>> char3int("=c3xiii")
    >>> char3int("=ciii")
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch; next field is at offset 1 but 4 expected

    >>> char3int("=cxxx@iii")

    Error:
    >>> char3int("cii")
    Traceback (most recent call last):
       ...
    ValueError: Buffer dtype mismatch, expected 'int' but got end in 'Char3Int.d'
    """
    cdef object obj = MockBuffer(fmt, sizeof(Char3Int))
    cdef object[Char3Int, ndim=1] buf = obj


def long_string(fmt):
    """
    >>> long_string("90198s")
    """
    cdef object obj = MockBuffer(fmt, sizeof(LongString))
    cdef object[LongString, ndim=1] buf = obj


def unpacked_struct(fmt):
    """
    Native formats:
    >>> unpacked_struct("ciZffciii")
    >>> unpacked_struct("@ci3fc3i")
    >>> unpacked_struct("@ciZffci2i")
    >>> unpacked_struct("ciZffT{ciii}")
    >>> unpacked_struct("cT{ifffc2i}i")
    >>> unpacked_struct("ciZffc3T{i}")
    >>> unpacked_struct("T{c}T{T{iZffT{ci}}}2T{T{i}}")
    """

    assert (sizeof(UnpackedStruct1) == sizeof(UnpackedStruct2)
            == sizeof(UnpackedStruct3) == sizeof(UnpackedStruct4))
    cdef object obj = MockBuffer(fmt, sizeof(UnpackedStruct1))
    cdef object[UnpackedStruct1, ndim=1] buf1 = obj
    cdef object[UnpackedStruct2, ndim=1] buf2 = obj
    cdef object[UnpackedStruct3, ndim=1] buf3 = obj
    cdef object[UnpackedStruct4, ndim=1] buf4 = obj

cdef struct ComplexTest:
    ComplexFloat a, b, c

def complex_test(fmt):
    """
    >>> complex_test("ZfZfZf")
    >>> complex_test("3Zf")
    >>> complex_test("6f")
    >>> complex_test("3T{Zf}")

    >>> complex_test("fZfZff")
    Traceback (most recent call last):
       ...
    ValueError: Buffer dtype mismatch, expected 'float' but got 'complex float' in 'ComplexFloat.imag'

    """
    cdef object obj = MockBuffer(fmt, sizeof(ComplexTest))
    cdef object[ComplexTest] buf1 = obj


def alignment_string(fmt, exc=None):
    """
    >>> alignment_string("@i")
    >>> alignment_string("%si" % current_endian)
    >>> alignment_string("%si" % other_endian, "X-endian buffer not supported on X-endian compiler")
    >>> alignment_string("=i")
    """
    cdef object[int] buf
    try:
        buf = MockBuffer(fmt, sizeof(int))
    except ValueError, e:
        msg = unicode(e).replace("Big", "X").replace("Little", "X").replace("big", "X").replace("little", "X")
        if msg != exc:
            print msg
            print "  is not equal to"
            print exc
        return
    if exc:
        print "fail"


def int_and_long_are_same():
    """
    >>> int_and_long_are_same()
    """
    cdef object[int] intarr
    cdef object[long] longarr
    if sizeof(int) == sizeof(long):
        intarr = MockBuffer("l", sizeof(int))
        longarr = MockBuffer("i", sizeof(int))

cdef struct MixedComplex:
    double real
    float imag

def mixed_complex_struct():
    """
    Triggering a specific execution path for this case.

    >>> mixed_complex_struct()
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch, expected 'double' but got 'complex double' in 'MixedComplex.real'
    """
    cdef object[MixedComplex] buf = MockBuffer("Zd",
        sizeof(MixedComplex))


cdef packed struct PackedSubStruct:
    char x
    int y

cdef struct UnpackedSubStruct:
    char x
    int y

cdef packed struct PackedStruct:
    char a
    int b
    PackedSubStruct sub

cdef struct PartiallyPackedStruct:
    char a
    int b
    PackedSubStruct sub

cdef packed struct PartiallyPackedStruct2:
    char a
    UnpackedSubStruct sub
    char b
    int c

def packed_struct(fmt):
    """
    Assuming int is four bytes:

    >>> packed_struct("^cici")
    >>> packed_struct("=cici")

    However aligned access won't work:

    >>> packed_struct("^c@i^ci")
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch; next field is at offset 4 but 1 expected

    >>> packed_struct("@cici")
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch; next field is at offset 4 but 1 expected

    """
    cdef object[PackedStruct] buf = MockBuffer(fmt, sizeof(PackedStruct))

def partially_packed_struct(fmt):
    """
    Assuming int is four bytes:

    >>> partially_packed_struct("^c@i^ci")
    >>> partially_packed_struct("@ci^ci")
    >>> partially_packed_struct("^c@i=ci")
    >>> partially_packed_struct("@ci=ci")
    >>> partially_packed_struct("ci^ci")
    >>> partially_packed_struct("ci=ci")

    Incorrectly aligned accesses won't work:

    >>> partially_packed_struct("^cici")
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch; next field is at offset 1 but 4 expected

    >>> partially_packed_struct("=cibi")
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch; next field is at offset 1 but 4 expected

    """
    cdef object[PartiallyPackedStruct] buf = MockBuffer(
        fmt, sizeof(PartiallyPackedStruct))

def partially_packed_struct_2(fmt):
    """
    Assuming int is four bytes:

    >>> partially_packed_struct_2("^ccxxxici")
    >>> partially_packed_struct_2("^ccxxxi^ci")
    >>> partially_packed_struct_2("c=cxxxi^ci")
    >>> partially_packed_struct_2("c^cxxxi^ci")
    >>> partially_packed_struct_2("c^cxxxi=ci")
    >>> partially_packed_struct_2("ccxxx^i@c^i")

    Incorrectly aligned accesses won't work:

    >>> partially_packed_struct_2("ccxxxici")
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch; next field is at offset 8 but 5 expected

    >>> partially_packed_struct_2("ccici")
    Traceback (most recent call last):
        ...
    ValueError: Buffer dtype mismatch; next field is at offset 4 but 5 expected

    """
    cdef object[PartiallyPackedStruct2] buf = MockBuffer(
        fmt, sizeof(PartiallyPackedStruct2))


cdef packed struct PackedStructWithCharArrays:
    float a
    int b
    char[5] c
    char[3] d


def packed_struct_with_strings(fmt):
    """
    >>> packed_struct_with_strings("T{f:a:i:b:5s:c:3s:d:}")
    """
    cdef object[PackedStructWithCharArrays] buf = MockBuffer(
        fmt, sizeof(PackedStructWithCharArrays))


ctypedef struct PackedStructWithArrays:
    double a[16]
    double b[16]
    double c

ctypedef struct UnpackedStructWithArrays:
    int a
    float b[8]
    float c
    unsigned long long d
    int e[5]
    int f
    int g
    double h[4]
    int i

ctypedef struct PackedStructWithNDArrays:
    double a
    double b[2][2]
    float c
    float d


def packed_struct_with_arrays(fmt):
    """
    >>> packed_struct_with_arrays("T{(16)d:a:(16)d:b:d:c:}")
    """

    cdef object[PackedStructWithArrays] buf = MockBuffer(
        fmt, sizeof(PackedStructWithArrays))


def unpacked_struct_with_arrays(fmt):
    """
    >>> if struct.calcsize('P') == 8:  # 64 bit
    ...     unpacked_struct_with_arrays("T{i:a:(8)f:b:f:c:Q:d:(5)i:e:i:f:i:g:xxxx(4)d:h:i:i:}")
    ... elif struct.calcsize('P') == 4:  # 32 bit
    ...     unpacked_struct_with_arrays("T{i:a:(8)f:b:f:c:Q:d:(5)i:e:i:f:i:g:(4)d:h:i:i:}")
    """

    cdef object[UnpackedStructWithArrays] buf = MockBuffer(
        fmt, sizeof(UnpackedStructWithArrays))


def packed_struct_with_ndarrays(fmt):
    """
    >>> packed_struct_with_ndarrays("T{d:a:(2,2)d:b:f:c:f:d:}")
    """

    cdef object[PackedStructWithNDArrays] buf = MockBuffer(
        fmt, sizeof(PackedStructWithNDArrays))


# TODO: empty struct
# TODO: Incomplete structs
# TODO: mixed structs



# ticket: t166

__doc__ = u"""
>>> l = None
>>> l.append(2)
Traceback (most recent call last):
AttributeError: 'NoneType' object has no attribute 'append'

"""

def append_to_none():
    """
    >>> append_to_none()
    Traceback (most recent call last):
    AttributeError: 'NoneType' object has no attribute 'append'
    """
    cdef list l = None
    l.append(2)
